using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using NLog;
using pdfforge.PDFCreator.Core.Ghostscript.OutputDevices;

//http://www.codeproject.com/KB/GDI-plus/ghostwrapper.aspx
//http://code.google.com/p/gouda/source/browse/trunk/Gouda/GhostscriptExports.cs?spec=svn2&r=2
//http://www.wewill.cn/n1926c8.aspx

namespace pdfforge.PDFCreator.Core.Ghostscript
{
    /// <summary>
    ///     Provides access to Ghostscript, either through DLL access or by calling the Ghostscript exe
    /// </summary>
    public class GhostScript
    {
        #region Api

        [DllImport("kernel32.dll", EntryPoint = "LoadLibraryA")]
        private static extern IntPtr LoadLibrary(string lpLibFileName);

        [DllImport("kernel32.dll", EntryPoint = "FreeLibrary", SetLastError = true)]
        private static extern bool FreeLibrary(IntPtr hModule);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern int gsapi_new_instance(out IntPtr pinstance, IntPtr callerHandle);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern int gsapi_init_with_args(IntPtr instance, Int32 argumentCount,
            [In, Out] string[] arguments);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern int gsapi_exit(IntPtr instance);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern void gsapi_delete_instance(IntPtr instance);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern int gsapi_set_stdio(IntPtr lngGsInstance, StdioCallBack gsdllStdin,
            StdioCallBack gsdllStdout, StdioCallBack gsdllStderr);

        [DllImport("gsdll32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        private static extern int gsapi_revision([In, Out] GhostscriptRevision revision, int len);

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public class GhostscriptRevision
        {
            public string product = "";
            public string copyright = "";
            public int revision = 0;
            public int revisiondate = 0;
        }

        #endregion

        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();
        public static string GhostscriptDll = "";
        public static string GhostscriptLib = "";

        private static GhostScript _instance;
        private IntPtr _dllHModule = IntPtr.Zero;
        public IntPtr ExtHandle = IntPtr.Zero;
        private IntPtr _intGsInstanceHandle;

        private GhostScript()
        {
        }

        public static bool UseGhostscriptExe { get; set; }

        public static GhostscriptVersion GhostscriptVersion { get; set; }
        public static event EventHandler<OutputEventArgs> Output;

        private int gsdll_stdin(IntPtr intGsInstanceHandle, IntPtr strz, int intBytes)
        {
            var buffer = new char[intBytes];
            Console.In.ReadBlock(buffer, 0, intBytes);
            strz = Marshal.StringToHGlobalAnsi(new string(buffer));
            return buffer.Length;
        }

        private int gsdll_stdout(IntPtr intGsInstanceHandle, IntPtr strz, int intBytes)
        {
            string result = Marshal.PtrToStringAnsi(strz, intBytes);
            Output(this, new OutputEventArgs(result));
            //Console.Out.Write(result);
            return intBytes;
        }

        private int gsdll_stderr(IntPtr intGsInstanceHandle, IntPtr strz, int intBytes)
        {
            return gsdll_stdout(intGsInstanceHandle, strz, intBytes);
        }

        /// <summary>
        ///     Get the instance Singelton
        /// </summary>
        /// <returns>The single Ghostscript instance</returns>
        public static GhostScript GetInstance()
        {
            if (_instance == null)
            {
                _instance = new GhostScript();
            }

            return _instance;
        }

        /// <summary>
        ///     Loads the Ghostscript Dll
        /// </summary>
        /// <returns>A handle pointer</returns>
        public IntPtr LoadDll()
        {
            return LoadDll(GhostscriptDll);
        }

        /// <summary>
        ///     Loads the Ghostscript Dll
        /// </summary>
        /// <param name="pathToGhostscriptDll">Full path to the Ghostscript Dll</param>
        /// <returns>A handle pointer</returns>
        private IntPtr LoadDll(string pathToGhostscriptDll)
        {
            _dllHModule = LoadLibrary(pathToGhostscriptDll);
            return _dllHModule;
        }

        private void NewInstance()
        {
            int intReturn = gsapi_new_instance(out _intGsInstanceHandle, ExtHandle);

            if (intReturn == 0)
                throw new Win32Exception("Error while instantiating ghostscript");

            StdioCallBack stdinCallback = gsdll_stdin;
            StdioCallBack stdoutCallback = gsdll_stdout;
            StdioCallBack stderrCallback = gsdll_stderr;
            intReturn = gsapi_set_stdio(_intGsInstanceHandle, stdinCallback, stdoutCallback, stderrCallback);

            if (intReturn == 0) 
                throw new Win32Exception("Error while assigning ghostscript stdio");
        }

        private void DeleteInstance()
        {
            gsapi_exit(_intGsInstanceHandle);
            gsapi_delete_instance(_intGsInstanceHandle);
        }

        /// <summary>
        ///     Returns the Ghostscript Revision of the current instance
        /// </summary>
        /// <param name="revision">Output parameter to take the revision</param>
        /// <returns>Zero, if no error occured, non-zero value otherwise</returns>
        public int Revision(out GhostscriptRevision revision)
        {
            int res = -1;
            var rev = new GhostscriptRevision();

            if (_dllHModule != IntPtr.Zero)
                res = gsapi_revision(rev, Marshal.SizeOf(rev));
            
            revision = rev;

            return res;
        }

        private bool RunGsCommandLine(IEnumerable<string> parameters)
        {
            string directory = Path.GetDirectoryName(GhostscriptDll) ?? String.Empty;
            
            // Start the child process.
            var p = new Process();
            // Redirect the output stream of the child process.
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.RedirectStandardOutput = true;
            p.StartInfo.FileName = Path.Combine(directory, "gswin32c.exe");
            p.StartInfo.CreateNoWindow = true;

            bool isFirst = true;
            var sb = new StringBuilder();

            foreach (string s in parameters)
            {
                string tmp = s;
                if (isFirst)
                {
                    isFirst = false;
                    continue;
                }

                if (tmp.Contains(" "))
                {
                    tmp = tmp.Replace("\"", "\\\"");
                    tmp = "\"" + tmp + "\"";
                }

                sb.Append(tmp);
                sb.Append(" ");
            }

            p.StartInfo.Arguments = sb.ToString();

            p.Start();
            // Do not wait for the child process to exit before
            // reading to the end of its redirected stream.
            // p.WaitForExit();
            // Read the output stream first and then wait.

            var sbout = new StringBuilder();

            while (!p.StandardOutput.EndOfStream)
            {
                var c = (char) p.StandardOutput.Read();
                sbout.Append(c);

                if ((c == ']') || (c == '\n'))
                {
                    RaiseOutputEvent(sbout.ToString());

                    sbout.Length = 0;
                }
            }

            if (sbout.Length > 0)
                RaiseOutputEvent(sbout.ToString());

            p.WaitForExit();

            return p.ExitCode == 0;
        }

        private void RaiseOutputEvent(string message)
        {
            if (Output != null)
            {
                Output(this, new OutputEventArgs(message));
            }
        }

        private bool Run(string[] parameters)
        {
            Logger.Debug("Ghostscript Parameters:\r\n" + String.Join("\r\n", parameters));

            if (UseGhostscriptExe)
            {
                return RunGsCommandLine(parameters);
            }
            else
            {
                NewInstance();

                gsapi_init_with_args(_intGsInstanceHandle, parameters.Length, parameters);
                DeleteInstance();
                return true;
            }
        }

        /// <summary>
        ///     Runs Ghostscript with the parameters specified by the OutputDevice
        /// </summary>
        /// <param name="output"></param>
        public bool Run(OutputDevice output)
        {
            var parameters = (List<string>) output.GetGhostScriptParameters();
            var success = Run(parameters.ToArray());

            string outputFolder = Path.GetDirectoryName(output.Job.OutputFilenameTemplate);

            if (outputFolder != null && !Directory.Exists(outputFolder))
                Directory.CreateDirectory(outputFolder);
            
            output.CollectTemporaryOutputFiles();

            return success;
        }

        /// <summary>
        ///     Unloads the Ghostscript Dll
        /// </summary>
        /// <returns>true, if no error occured</returns>
        public bool FreeDll()
        {
            bool res = false;
            if (_dllHModule != IntPtr.Zero)
                res = FreeLibrary(_dllHModule);
            return res;
        }

        ~GhostScript()
        {
            FreeDll();
        }

        private delegate int StdioCallBack(IntPtr handle, IntPtr strptr, int count);
    }

    public class OutputEventArgs : EventArgs
    {
        public OutputEventArgs(string output)
        {
            Output = output;
        }

        public string Output { get; private set; }
    }
}