﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using Microsoft.Win32;

namespace pdfforge.PDFCreator.Core.Ghostscript
{
    /// <summary>
    ///     Specifies information on Ghostscript installations
    /// </summary>
    public class GhostscriptVersion
    {
        /// <summary>
        ///     Creates a new GhostscriptVersion object
        /// </summary>
        /// <param name="version">Ghostscript version string</param>
        /// <param name="dllPath">Full path of the gsdll32.dll</param>
        /// <param name="libFolder">Full path of the Lib folder</param>
        private GhostscriptVersion(string version, string dllPath, string libFolder)
        {
            Version = version;
            DllPath = dllPath;
            LibPaths = libFolder;
        }

        public string Version { get; private set; }
        public string DllPath { get; private set; }
        public string LibPaths { get; private set; }

        /// <summary>
        ///     Loads the settings into the Ghostscript class
        /// </summary>
        /// <returns>true, if successful</returns>
        public bool LoadGhostscript()
        {
            GhostScript.GhostscriptRevision rev;
            GhostScript.GhostscriptVersion = this;

            GhostScript.GhostscriptDll = DllPath;
            GhostScript.GhostscriptLib = LibPaths;
            GhostScript gs = GhostScript.GetInstance();

            if (GhostScript.UseGhostscriptExe)
                return true;

            gs.LoadDll();
            int res = gs.Revision(out rev);
            return res == 0;
        }

        /// <summary>
        ///     Get the internal instance if it exists, otherwise the most recent installed instance
        /// </summary>
        /// <returns>The best matching Ghostscript instance</returns>
        public static GhostscriptVersion GetBestGhostscriptInstance(string minGsVersion)
        {
            IList<GhostscriptVersion> versions = GetInstalledInstances(minGsVersion);

            if (versions.Count == 0)
                return null;

            return versions[0];
        }

        /// <summary>
        ///     Finds all Ghostscript instances installed in the registry together with internal instances.
        /// </summary>
        /// <returns>A list with all Ghostscript instances</returns>
        public static IList<GhostscriptVersion> GetInstalledInstances(string minGsVersion)
        {
            var versions = FindRegistryInstances(minGsVersion);

            GhostscriptVersion internalInstance = FindInternalInstance();

            if (internalInstance != null)
                versions.Add(internalInstance);

            versions.Reverse();

            return versions;
        }

        /// <summary>
        ///     Search for Ghostscript instances in the application folder
        /// </summary>
        /// <returns>A GhostscriptVersion if an internal instance exists, null otherwise</returns>
        private static GhostscriptVersion FindInternalInstance()
        {
            string exePath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase) ?? "";

            if (exePath.StartsWith(@"file:\"))
                exePath = exePath.Substring(6);

            string[] paths = {Path.Combine(exePath, "Ghostscript"), Path.Combine(exePath, @"..\..\Ghostscript")};

            foreach (string path in paths)
            {
                string dllPath = Path.Combine(path, @"Bin\gsdll32.dll");
                string libPath = Path.Combine(path, @"Bin") + ';' + Path.Combine(path, @"Lib") + ';' +
                                 Path.Combine(path, @"Fonts");

                if (File.Exists(dllPath))
                {
                    return new GhostscriptVersion("<internal>", dllPath, libPath);
                }
            }

            return null;
        }

        /// <summary>
        ///     Find all Ghostscript instances installed in the registry
        /// </summary>
        /// <returns>A list with all instances</returns>
        private static List<GhostscriptVersion> FindRegistryInstances(string minGsVersion)
        {
            var versions = new List<GhostscriptVersion>();

            string gsKey;
            if (OsHelper.Is64BitOperatingSystem)
                gsKey = "SOFTWARE\\Wow6432Node\\GPL Ghostscript";
            else
                gsKey = "SOFTWARE\\GPL Ghostscript";

            RegistryKey gsMainKey = Registry.LocalMachine.OpenSubKey(gsKey);

            if (gsMainKey == null)
                return versions;

            foreach (string subkey in gsMainKey.GetSubKeyNames())
            {
                GhostscriptVersion v = IsGhostscriptInstalled(subkey);

                if ((v != null) && (string.Compare(v.Version, minGsVersion, StringComparison.Ordinal) >= 0))
                    versions.Add(v);
            }

            return versions;
        }

        /// <summary>
        ///     Check if Ghostscript is installed with a given version. It does a lookup in the registry and checks if the paths
        ///     exist.
        /// </summary>
        /// <param name="version">Name of the version to check, i.e. "9.05"</param>
        /// <returns>A GhostscriptVersion object if a version has been found, null otherwise</returns>
        private static GhostscriptVersion IsGhostscriptInstalled(string version)
        {
            string gsKey;
            if (OsHelper.Is64BitOperatingSystem)
                gsKey = "SOFTWARE\\Wow6432Node\\GPL Ghostscript\\" + version;
            else
                gsKey = "SOFTWARE\\GPL Ghostscript\\" + version;

            try
            {
                RegistryKey myKey = Registry.LocalMachine.OpenSubKey(gsKey);
                if (myKey == null)
                    return null;

                var gsDll = (string) myKey.GetValue("GS_DLL");
                var gsLib = (string) myKey.GetValue("GS_LIB");

                myKey.Close();
                if (File.Exists(gsDll))
                {
                    return new GhostscriptVersion(version, gsDll, gsLib);
                }
            }
            catch
            {
                return null;
            }

            return null;
        }
    }
}