﻿using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text;
using pdfforge.DataStorage;
using pdfforge.DataStorage.Storage;
using pdfforge.PDFCreator.Core.Settings;

namespace pdfforge.PDFCreator.Core.Jobs
{
    /// <summary>
    ///     The JobInfo class holds all information about the job, like the source files, metadata and such.
    /// </summary>
    public class JobInfo : IJobInfo
    {
        //public ApplicationSettings AppSettings;
        public string InfFile { get; private set; }
        public IList<SourceFileInfo> SourceFiles { get; private set; }
        public Metadata Metadata { get; set; }

        public JobInfo()
        {
            SourceFiles = new List<SourceFileInfo>();
        }

        /// <summary>
        ///     Creates a JobInfo based on the given Inf file
        /// </summary>
        /// <param name="infFile">full path to the Inf file to use</param>
        /// <param name="appSettings">application settings to use for initialization</param>
        public JobInfo(string infFile, ApplicationSettings appSettings)
        {
            SourceFiles = new List<SourceFileInfo>();

            InfFile = infFile;
            Data infData = Data.CreateDataStorage();
            var ini = new IniStorage(Encoding.GetEncoding("Unicode"));
            ini.SetData(infData);
            ini.ReadData(infFile);

            foreach (string section in infData.GetSections())
            {
                SourceFileInfo sfi = SourceFileInfo.ReadSourceFileInfo(infFile, infData, section);
                if (sfi != null)
                    SourceFiles.Add(sfi);
            }

            Metadata = new Metadata();

            if (SourceFiles.Count > 0)
            {
                Metadata.Author = SourceFiles[0].Author;
                Metadata.Title = GetCleanTitle(appSettings);
            }
        }

        public void SaveInf()
        {
            Data infData = Data.CreateDataStorage();
            var ini = new IniStorage(Encoding.GetEncoding("Unicode"));
            ini.SetData(infData);

            int sectionId = 0;

            foreach (var sourceFileInfo in SourceFiles)
            {
                sourceFileInfo.WriteSourceFileInfo(infData, sectionId.ToString(CultureInfo.InvariantCulture) + "\\");

                sectionId++;
            }

            ini.WriteData(InfFile);
        }

        public void SaveInf(string filename)
        {
            InfFile = filename;
            SaveInf();
        }

        /// <summary>
        /// Deletes the inf file and optionally the source files
        /// <param name="includeSourceFiles">If true, the source filesd will be deleted as well</param>
        /// </summary>
        public void Delete(bool includeSourceFiles)
        {
            if (includeSourceFiles)
            {
                foreach (var sourceFileInfo in SourceFiles)
                {
                    try
                    {
                        File.Delete(sourceFileInfo.Filename);
                    }
                    catch (IOException)
                    {
                    }
                }
                SourceFiles.Clear();
            }
            
            try
            {
                if (!string.IsNullOrEmpty(InfFile))
                    File.Delete(InfFile);
            }
            catch (IOException)
            {
            }
            InfFile = null;
        }

        public void Merge(IJobInfo jobInfo)
        {
            foreach (var sourceFile in jobInfo.SourceFiles)
            {
                SourceFiles.Add(sourceFile);
            }

            jobInfo.Delete(false);
        }

        private string GetCleanTitle(ApplicationSettings appSettings)
        {
            string title = SourceFiles[0].DocumentTitle;

            foreach (TitleReplacement tr in appSettings.TitleReplacement)
            {
                title = title.Replace(tr.Search, tr.Replace);
            }

            return title;
        }
    }
}