﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Data;
using NUnit.Framework;
using PDFCreator.Test.ViewModels.Helper;
using pdfforge.DynamicTranslator;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.ViewModels;
using Rhino.Mocks;

namespace PDFCreator.Test.ViewModels
{
    [TestFixture]
    class ApplicationSettingsViewModelTest
    {
        private ApplicationSettings _applicationSettings;
        private ConversionProfile _profile1;
        private ConversionProfile _profile2;

        private const string ExistingPrinter = "PDFCreator";
        private const string UnknownPrinter = "UnknownPrinter";

        [SetUp]
        public void SetUp()
        {
            _profile1 = new ConversionProfile()
            {
                Name = "Profile1",
                Guid = "Profile1Guid"
            };

            _profile2 = new ConversionProfile()
            {
                Name = "Profile1",
                Guid = "Profile1Guid"
            };

            _applicationSettings = new ApplicationSettings();
            _applicationSettings.PrinterMappings = new[] { new PrinterMapping(ExistingPrinter, _profile1.Guid), new PrinterMapping(UnknownPrinter, _profile2.Guid) };
        }

        [Test]
        public void EmptyViewModel_SettingApplicationSettings_RaisesPropertyChanged()
        {
            var eventStub = MockRepository.GenerateStub<IEventHandler<PropertyChangedEventArgs>>();
            var asViewModel = new ApplicationSettingsViewModel();
            asViewModel.PropertyChanged += eventStub.OnEventRaised;
            var propertyListener = new PropertyChangedListenerMock(asViewModel, "ApplicationSettings");

            asViewModel.ApplicationSettings = new ApplicationSettings();

            Assert.IsTrue(propertyListener.WasCalled);
        }

        [Test]
        public void EmptyViewModel_SettingPdfCreatorPrinters_RaisesPropertyChanged()
        {
            var eventStub = MockRepository.GenerateStub<IEventHandler<PropertyChangedEventArgs>>();
            var asViewModel = new ApplicationSettingsViewModel();
            asViewModel.PropertyChanged += eventStub.OnEventRaised;
            var propertyListener = new PropertyChangedListenerMock(asViewModel, "PdfCreatorPrinters");

            asViewModel.PdfCreatorPrinters = new List<string>();

            Assert.IsTrue(propertyListener.WasCalled);
        }

        [Test]
        public void EmptyViewModel_SettingLanguages_RaisesPropertyChanged()
        {
            var eventStub = MockRepository.GenerateStub<IEventHandler<PropertyChangedEventArgs>>();
            var asViewModel = new ApplicationSettingsViewModel();
            asViewModel.PropertyChanged += eventStub.OnEventRaised;
            var propertyListener = new PropertyChangedListenerMock(asViewModel, "Languages");

            asViewModel.Languages = new List<Language>();

            Assert.IsTrue(propertyListener.WasCalled);
        }

        [Test]
        public void RenamePrinterCommand_WithNonexistingPrinter_NotExecutable()
        {
            var asViewModel = new ApplicationSettingsViewModel(_applicationSettings, null, new List<ConversionProfile>(), () => new[] { ExistingPrinter, "Something" }, null);

            var view = CollectionViewSource.GetDefaultView(asViewModel.PrinterMappings);
            view.MoveCurrentToLast();

            Assert.IsFalse(asViewModel.RenamePrinterCommand.CanExecute(null));
        }

        [Test]
        public void RenamePrinterCommand_WithValidPrinter_IsExecutable()
        {
            var asViewModel = new ApplicationSettingsViewModel(_applicationSettings, null, new List<ConversionProfile>(), () => new[] { ExistingPrinter, "Something" }, null);

            var view = CollectionViewSource.GetDefaultView(asViewModel.PrinterMappings);
            view.MoveCurrentToFirst();

            Assert.IsTrue(asViewModel.RenamePrinterCommand.CanExecute(null));
        }

        [Test]
        public void DeletePrinterCommand_WithNonexistingPrinter_NotExecutable()
        {
            var asViewModel = new ApplicationSettingsViewModel(_applicationSettings, null, new List<ConversionProfile>(), () => new[] { ExistingPrinter, "Something" }, null);

            var view = CollectionViewSource.GetDefaultView(asViewModel.PrinterMappings);
            view.MoveCurrentToLast();

            Assert.IsFalse(asViewModel.DeletePrinterCommand.CanExecute(null));
        }

        [Test]
        public void DeletePrinterCommand_WithValidPrinter_IsExecutable()
        {
            var asViewModel = new ApplicationSettingsViewModel(_applicationSettings, null, new List<ConversionProfile>(), () => new[] { ExistingPrinter, "Something" }, null);

            var view = CollectionViewSource.GetDefaultView(asViewModel.PrinterMappings);
            view.MoveCurrentToFirst();

            Assert.IsTrue(asViewModel.DeletePrinterCommand.CanExecute(null));
        }

        [Test]
        public void EmptyViewModel_WithEmptyProfileList_ContainsLastUsedMappingProfile()
        {
            var asViewModel = new ApplicationSettingsViewModel(new ApplicationSettings(), new ApplicationProperties(), new ConversionProfile[]{}, () => new string[]{}, new Language[]{} );

            Assert.AreEqual("", asViewModel.PrinterMappingProfiles.First().Guid);
        }

        #region TitleMoveUpCommand
        [Test]
        public void TitleReplacementMoveUp_WithNullList_CannotExecute()
        {
            var asViewModel = new ApplicationSettingsViewModel(null, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleMoveUpCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveUp_WithEmptyList_CannotExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new TitleReplacement[] {};

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleMoveUpCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveUp_WithFirstElement_CannotExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] {new TitleReplacement("a", "b"), new TitleReplacement("c", "d") };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentTo(asViewModel.TitleReplacements[0]);

            Assert.IsFalse(asViewModel.TitleMoveUpCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveUp_WithSecondElement_CanExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentTo(asViewModel.TitleReplacements[1]);

            Assert.IsTrue(asViewModel.TitleMoveUpCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveUp_WithSecondElement_MovesToFirst()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentTo(asViewModel.TitleReplacements[1]);

            asViewModel.TitleMoveUpCommand.Execute(null);

            Assert.AreEqual("c", settings.TitleReplacement[0].Search);
        }

        [Test]
        public void TitleReplacementMoveUp_CurrentItemChanged_RaisesCommandCanExecuteChanged()
        {
            bool wasRaised = false;
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();
            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            asViewModel.TitleMoveUpCommand.CanExecuteChanged += (sender, args) => wasRaised = true;

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToLast();

            Assert.IsTrue(wasRaised);
        }

        [Test]
        public void TitleReplacementMoveUp_WithSecondElement_RaisesCommandCanExecuteChanged()
        {
            bool wasRaised = false;
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToLast();

            asViewModel.TitleMoveUpCommand.CanExecuteChanged += (sender, args) => wasRaised = true;

            asViewModel.TitleMoveUpCommand.Execute(null);

            Assert.IsTrue(wasRaised);
        }
        #endregion TitleMoveUpCommand

        #region TitleMoveDownCommand
        [Test]
        public void TitleReplacementMoveDown_WithNullList_CannotExecute()
        {
            var asViewModel = new ApplicationSettingsViewModel(null, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleMoveDownCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveDown_WithEmptyList_CannotExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new TitleReplacement[] { };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleMoveDownCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveDown_WithLastElement_CannotExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToLast();

            Assert.IsFalse(asViewModel.TitleMoveDownCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveDown_WithFirstElement_CanExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToFirst();

            Assert.IsTrue(asViewModel.TitleMoveDownCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementMoveDown_WithFirstElement_MovesToSecond()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToFirst();

            asViewModel.TitleMoveDownCommand.Execute(null);

            Assert.AreEqual("a", settings.TitleReplacement[1].Search);
        }

        [Test]
        public void TitleReplacementMoveDown_CurrentItemChanged_RaisesCommandCanExecuteChanged()
        {
            bool wasRaised = false;
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();
            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            asViewModel.TitleMoveDownCommand.CanExecuteChanged += (sender, args) => wasRaised = true;

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToLast();

            Assert.IsTrue(wasRaised);
        }

        [Test]
        public void TitleReplacementMoveDown_WithFirstElement_RaisesCommandCanExecuteChanged()
        {
            bool wasRaised = false;
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToFirst();

            asViewModel.TitleMoveDownCommand.CanExecuteChanged += (sender, args) => wasRaised = true;

            asViewModel.TitleMoveDownCommand.Execute(null);

            Assert.IsTrue(wasRaised);
        }

        #endregion TitleMoveDownCommand

        #region TitleAddCommand

        [Test]
        public void TitleReplacementAdd_WithEmptyList_CanExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new TitleReplacement[] { };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsTrue(asViewModel.TitleAddCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementAdd_WithEmptyList_InsertsOneElement()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new TitleReplacement[] { }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            asViewModel.TitleAddCommand.Execute(null);

            Assert.AreEqual(1, settings.TitleReplacement.Count);
        }

        [Test]
        public void TitleReplacementAdd_WithNonEmptyList_NewElementIsCurrent()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);
            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);

            asViewModel.TitleAddCommand.Execute(null);

            Assert.AreEqual(2, cv.CurrentPosition);
        }

        #endregion

        #region TitleDeleteCommand

        [Test]
        public void TitleReplacementDelete_WithNullList_CannotExecute()
        {
            var asViewModel = new ApplicationSettingsViewModel(null, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleDeleteCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementDelete_WithEmptyList_CannotExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new TitleReplacement[] { };

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsFalse(asViewModel.TitleDeleteCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementDelete_WithNonEmptyList_CanExecute()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            Assert.IsTrue(asViewModel.TitleDeleteCommand.CanExecute(null));
        }

        [Test]
        public void TitleReplacementDelete_AfterExecute_ElementIsRemoved()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var deletedElement = settings.TitleReplacement[0];

            asViewModel.TitleDeleteCommand.Execute(null);

            Assert.IsFalse(asViewModel.TitleReplacements.Contains(deletedElement));
        }

        [Test]
        public void TitleReplacementDelete_AfterExecute_OtherElementStillThere()
        {
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();

            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            var otherElement = settings.TitleReplacement[1];

            asViewModel.TitleDeleteCommand.Execute(null);

            Assert.IsTrue(asViewModel.TitleReplacements.Contains(otherElement));
        }

        [Test]
        public void TitleReplacementDelete_CurrentItemChanged_RaisesCommandCanExecuteChanged()
        {
            bool wasRaised = false;
            var settings = new ApplicationSettings();
            settings.TitleReplacement = new[] { new TitleReplacement("a", "b"), new TitleReplacement("c", "d") }.ToList();
            var asViewModel = new ApplicationSettingsViewModel(settings, null, new List<ConversionProfile>(), () => new[] { "PDFCreator", "Something" }, null);

            asViewModel.TitleDeleteCommand.CanExecuteChanged += (sender, args) => wasRaised = true;

            var cv = CollectionViewSource.GetDefaultView(asViewModel.TitleReplacements);
            cv.MoveCurrentToLast();

            Assert.IsTrue(wasRaised);
        }

        #endregion
    }
}
