﻿using System;
using System.Diagnostics;
using System.IO;
using System.Security;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using Microsoft.Win32;
using NLog;
using pdfforge.PDFCreator.Communication;
using pdfforge.PDFCreator.Core.Jobs;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Forms;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.Threading;
using pdfforge.PDFCreator.Utilities;
using pdfforge.PDFCreator.Utilities.IO;
using pdfforge.PDFCreator.ViewModels;
using pdfforge.PDFCreator.Views;

namespace pdfforge.PDFCreator
{
    public partial class App : Application
    {
        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        public App()
        {
            InitializeComponent();

            System.Windows.Forms.Application.EnableVisualStyles();
        }

        private void App_OnStartup(object sender, StartupEventArgs e)
        {
            GlobalMutex globalMutex = new GlobalMutex("PDFCreator-137a7751-1070-4db4-a407-83c1625762c7");
            globalMutex.Acquire();
            
            AppDomain.CurrentDomain.UnhandledException += CurrentDomain_UnhandledException;

            try
            {
                // Setting up Variables from Volatile Environment, as this does not happen in the pdfcmon
                SetVolatileEnvironment();

                Thread.CurrentThread.Name = "ProgramThread";
                var commandlineParser = new CommandLineParser(e.Args);

                if (TranslationUtil.TranslationPath == null)
                {
                    MessageBox.Show(@"Could not find any translation. Please reinstall PDFCreator.",
                        @"Translations missing");
                    Shutdown(1);
                }

                string newJob = null;
                SettingsHelper.Init();

                if (commandlineParser.HasArgument("PrintFile"))
                {
                    Logger.Info("Launched printjob with PrintFile command.");
                    LaunchPrintJob(commandlineParser);

                    Shutdown();
                    return;
                }

                string infFile = null;

                if (commandlineParser.HasArgument("InfoDataFile"))
                {
                    Logger.Info("Launched PDFCreator with InfoDataFile parameter.");
                    infFile = commandlineParser.GetArgument("InfoDataFile");
                }

                if (infFile == null && commandlineParser.HasArgument("PIF"))
                {
                    Logger.Info("Launched PDFCreator with PIF parameter.");
                    infFile = commandlineParser.GetArgument("PIF");
                }

                if (!String.IsNullOrEmpty(infFile))
                {
                    if (File.Exists(infFile))
                    {
                        newJob = infFile;
                        Logger.Debug("Added \"" + infFile + "\" as new Job.");
                    }
                    else
                    {
                        Logger.Error("No file in InfoDataFile argument or file does not exist.");
                        return;
                    }
                }

                Logger.Debug("Starting PDFCreator");
                JobInfoQueue.Init();

                if (!JobInfoQueue.Instance.SpoolFolderIsAccessible())
                {
                    if (TryRepairSpoolPath() == false)
                    {
                        return;
                    }
                }

                if (newJob != null)
                {
                    if (!Path.GetFullPath(newJob).StartsWith(JobInfoQueue.Instance.SpoolFolder))
                    {
                        Logger.Debug(
                            "JobInfo file from printer is not in our spool folder ({0}) - we'll move it there",
                            JobInfoQueue.Instance.SpoolFolder);
                        newJob = MoveSpoolFile(newJob, JobInfoQueue.Instance.SpoolFolder,
                            SettingsHelper.Settings.ApplicationSettings);
                    }
                }

                if (e.Args.Length > 0)
                {
                    Logger.Info("Command Line parameters: " + String.Join(" ", e.Args));
                }
                try
                {
                    int retry = 0;
                    bool success = false;

                    // Make n Attempts: Look if a pipe server exists, if so, send a message. If that fails, retry (and maybe do the job yourself)
                    while (!success && retry++ <= 5)
                    {
                        Logger.Debug("Starting attempt {0}: ", retry);
                        if (PipeServer.SessionServerInstanceRunning(ThreadManager.PipeName))
                        {
                            success = TrySendPipeMessage(newJob);
                            Logger.Debug("TrySendPipeMessage: " + success);
                        }
                        else
                        {
                            success = TryStartApplication(commandlineParser, newJob);
                            Logger.Debug("TryStartApplication: " + success);
                        }
                    }

                    Logger.Debug("Waiting for all threads to finish");
                    ThreadManager.Instance.WaitForThreadsAndShutdown(this);
                }
                catch (Exception ex)
                {
                    Logger.Error(string.Format("There was an error while starting PDFCreator:\r\n{0}\r\n{1}", ex.Message,
                        ex));

                    string message = TranslationUtil.GetFormattedTranslation("Program", "ErrorWhileStarting",
                        "There was an error while starting PDFCreator:\r\n{0}\r\n{1}", ex.Message, ex);
                    const string caption = "PDFCreator";
                    MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                }
            }
            finally
            {
                globalMutex.Release();
                Logger.Debug("Ending PDFCreator");
                Shutdown();
            }
        }

        private bool TryRepairSpoolPath()
        {
            Logger.Error("The spool folder is not accessible due to a permission problem. PDFCreator will not work this way");

            string tempFolder = Path.GetFullPath(Path.Combine(JobInfoQueue.Instance.SpoolFolder, ".."));
            string username = Environment.UserName;

            Logger.Debug("Username is {0}", username);

            string title = TranslationUtil.GetTranslation("Application", "SpoolFolderAccessDenied", "Access Denied");
            string message = TranslationUtil.GetFormattedTranslation("Application", "SpoolFolderAskToRepair", "The temporary path where PDFCreator stores the print jobs can't be accessed. This is a configuration problem on your machine and needs to be fixed. Do you want PDFCreator to attempt repairing it?\r\nYour spool folder is: {0}", tempFolder);

            Logger.Debug("Asking to start repair..");
            if (MessageWindow.ShowTopMost(message, title, MessageWindowButtons.YesNo, MessageWindowIcon.Exclamation) == MessageWindowResponse.Yes)
            {
                string repairToolPath = AppDomain.CurrentDomain.BaseDirectory;
                repairToolPath = Path.Combine(repairToolPath, "RepairFolderPermissions.exe");

                string repairToolParameters = string.Format("\"{0}\" \"{1}\"", username, tempFolder);

                Logger.Debug("RepairTool path is: {0}", repairToolPath);
                Logger.Debug("Parameters: {0}", repairToolParameters);

                if (!File.Exists(repairToolPath))
                {
                    Logger.Error("RepairFolderPermissions.exe does not exist!");
                    title = TranslationUtil.GetTranslation("Application", "RepairToolNotFound", "RepairTool not found");
                    message = TranslationUtil.GetFormattedTranslation("Application", "SetupFileMissing",
                        "An important PDFCreator file is missing ('{0}'). Please reinstall PDFCreator!",
                        Path.GetFileName(repairToolPath));

                    MessageWindow.ShowTopMost(message, title, MessageWindowButtons.OK, MessageWindowIcon.Error);
                    return false;
                }

                Logger.Debug("Starting RepairTool...");
                var result = ShellExecuteHelper.RunAsAdmin(repairToolPath, repairToolParameters);
                Logger.Debug("Done: {0}", result.ToString());
            }

            Logger.Debug("Now we'll check again, if the spool folder is not accessible");
            if (!JobInfoQueue.Instance.SpoolFolderIsAccessible())
            {
                Logger.Info("The spool folder could not be repaired.");
                title = TranslationUtil.GetTranslation("Application", "SpoolFolderAccessDenied", "Access Denied");
                message = TranslationUtil.GetFormattedTranslation("Application", "SpoolFolderUnableToRepair", "PDFCreator was not able to repair your spool folder. Please contact your administrator or the support to assist you in changing the permissions of the path '{0}'.", tempFolder);

                MessageWindow.ShowTopMost(message, title, MessageWindowButtons.OK, MessageWindowIcon.Exclamation);
                return false;
            }

            Logger.Info("The spool folder was repaired successfully");

            return true;
        }

        private string MoveSpoolFile(string infFile, string spoolFolder, ApplicationSettings applicationSettings)
        {
            JobInfo ji = new JobInfo(infFile, applicationSettings);

            string jobName = Path.GetFileNameWithoutExtension(infFile);
            string jobFolder = Path.Combine(spoolFolder, jobName);

            jobFolder = new UniqueDirectory(jobFolder).MakeUniqueDirectory();
            Directory.CreateDirectory(jobFolder);

            foreach (var sourceFile in ji.SourceFiles)
            {
                string targetFile = Path.Combine(jobFolder, Path.GetFileName(sourceFile.Filename));
                File.Move(sourceFile.Filename, targetFile);
                sourceFile.Filename = Path.GetFileName(sourceFile.Filename);
            }

            var newInfFile = Path.Combine(jobFolder, jobName + ".inf");
            ji.SaveInf(newInfFile);

            File.Delete(infFile);

            return newInfFile;
        }

        private bool TryStartApplication(CommandLineParser commandlineParser, string newJob)
        {
            try
            {
                Logger.Debug("Starting pipe server");
                ThreadManager.Instance.StartPipeServerThread();

                Logger.Debug("Reloading settings");
                // Settings may have changed as this may have not been the only instance till now
                SettingsHelper.ReloadSettings();

                ThreadManager.Instance.PipeServer.OnNewMessage += PipeServer_OnNewMessage;

                JobRunner.Instance.RegisterJobInfoQueueHandler();

                if (commandlineParser.HasArgument("ManagePrintJobs"))
                {
                    JobRunner.Instance.ManagePrintJobs();
                }

                Logger.Debug("Finding spooled jobs");
                JobInfoQueue.Instance.FindSpooledJobs();

                if (newJob != null)
                {
                    Logger.Debug("Adding new job");
                    JobInfoQueue.Instance.Add(newJob);
                }
                else
                {
                    if (commandlineParser.HasArgument("Debug"))
                        TestForm.ShowTestForm();

                    Logger.Debug("Starting main form");
                    ThreadManager.Instance.StartMainWindowThread();
                }

                Logger.Debug("Starting update check thread");
                StartUpdateCheck();

                Logger.Debug("Starting Cleanup thread");
                ThreadManager.Instance.StartCleanUpThread();

                return true;
            }
            catch (Exception ex)
            {
                Logger.Error("There was an error while starting the application: {0}", ex);
                return false;
            }
        }

        void PipeServer_OnNewMessage(object sender, MessageEventArgs e)
        {
            if (e.Message.StartsWith("ReloadSettings|"))
            {
                Logger.Info("Pipe Command: Reloading settings");
                SettingsHelper.ReloadSettings();
            }
        }

        private static void StartUpdateCheck()
        {
            UpdateManager.Instance.Initialize(SettingsHelper.Settings);
            UpdateManager.Instance.NewVersionEvent += UpdateManager.LaunchNewUpdateForm;
            UpdateManager.Instance.FinishedEvent += StoreNextUpdate;
            UpdateManager.Instance.UpdateProcedure(true);
        }

        private static void StoreNextUpdate(object sender, UpdateEventArgs eventArgs)
        {
            SettingsHelper.Settings.ApplicationProperties.NextUpdate = UpdateManager.NextUpdate;
        }

        private static bool TrySendPipeMessage(string newJob)
        {
            Logger.Debug("Found another running instance of PDFCreator, so we send our data there");

            string message;

            if (newJob != null)
            {
                message = "NewJob|" + newJob;
            }
            else
            {
                message = "ShowMain|";
            }

            PipeClient pipeClient = PipeClient.CreateSessionPipeClient(ThreadManager.PipeName);

            if (pipeClient.SendMessage(message))
            {
                Logger.Debug("Pipe message successfully sent");
                return true;
            }

            Logger.Warn("There was an error while communicating through the pipe");
            return false;
        }

        private static void LaunchPrintJob(CommandLineParser commandlineParser)
        {
            string filename = commandlineParser.GetArgument("PrintFile");

            if (String.IsNullOrEmpty(filename))
            {
                Logger.Error("PrintFile Parameter has no argument");
                return;
            }

            if (!File.Exists(filename))
            {
                Logger.Error(string.Format("The file \"{0}\" does not exist!", filename));
                return;
            }

            var printFileHelper = new PrintFileHelper();

            if (!printFileHelper.AddFile(filename))
            {
                Logger.Warn(string.Format("The file \"{0}\" is not printable!", filename));
                return;
            }

            if (!printFileHelper.PrintAll())
                Logger.Error(string.Format("The file \"{0}\" could not be printed!", filename));
        }

        

        void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            var ex = (Exception) e.ExceptionObject;
            Logger.FatalException(string.Format("Uncaught exception, IsTerminating: {0}", e.IsTerminating), ex);
            ErrorHelper.ShowErrorWindowInNewProcess(ex);
        }

        private void Application_DispatcherUnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            Logger.ErrorException("Uncaught exception in WPF thread", e.Exception);

            e.Handled = true;

            bool terminateRequested;
            ErrorHelper.ShowErrorWindow(e.Exception, out terminateRequested);

            if (terminateRequested)
            {
                Current.Shutdown(1);
            }
        }

        private static void SetVolatileEnvironment()
        {
            try
            {
                int sessionId = Process.GetCurrentProcess().SessionId;

                Environment.SetEnvironmentVariable("SESSION_ID", sessionId.ToString());

                RegistryKey key = Registry.CurrentUser.OpenSubKey("Volatile Environment\\" + sessionId);

                if (key != null)
                {
                    foreach (var name in key.GetValueNames())
                    {
                        if (string.IsNullOrWhiteSpace(name))
                            continue;

                        var value = key.GetValue(name);

                        if (value == null || key.GetValueKind(name) != RegistryValueKind.String)
                            continue;

                        try
                        {
                            Environment.SetEnvironmentVariable(name, value.ToString());
                        }
                        catch (ArgumentException)
                        {

                        }
                        catch (SecurityException)
                        {

                        }
                    }
                }
            }
            catch
            {
                return;
            }
        }
    }
}
