﻿using System;
using System.Runtime.InteropServices;
using System.Threading;
using NLog;
using pdfforge.PDFCreator.Communication;
using pdfforge.PDFCreator.Core.Jobs;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.Threading;

namespace pdfforge.PDFCreator.COM
{
    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
    [Guid("3803F46C-F5AA-4B86-8B9C-6EFFAC9CDCFA")]
    public interface IQueue
    {
        void Initialize();
        bool WaitForJob(int timeOut);
        bool WaitForJobs(int jobCount, int timeOut);
        int Count { get; }
        PrintJob NextJob { get; }
        PrintJob GetJobByIndex(int jobIndex);
        void MergeJobs(PrintJob job1, PrintJob job2);
        void MergeAllJobs();
        void ReleaseCom();
    }


    [ComVisible(true)]
    [ClassInterface(ClassInterfaceType.AutoDispatch)]
    [Guid("66A9CAB1-404A-4918-8DE2-29C26B9B271E")]
    [ProgId("PDFCreatorBeta.JobQueue")]
    public class Queue : IQueue
    {
        private static readonly Logger COMLogger = LogManager.GetCurrentClassLogger();
        private static bool _isServerInstanceRunning;
        private readonly AutoResetEvent _newJobResetEvent = new AutoResetEvent(false);
        private JobInfoQueue _comJobInfoQueue;
        private readonly ThreadPool _threadPool = ThreadPool.Instance;
        private bool _isComActive;

        /// <summary>
        ///     If another instance of PDFCreator is running, throws an error, otherwise starts an instance itself
        /// </summary>
        public Queue()
        {
            if (PipeServer.SessionServerInstanceRunning(ThreadManager.PipeName))
                _isServerInstanceRunning = true;
        }

        /// <summary>
        ///     Initializes the essential components like JobInfoQueue for the COM object
        /// </summary>
        public void Initialize()
        {
            SettingsHelper.Init();

            if(_isServerInstanceRunning)
                throw new COMException("Access forbidden. An instance of PDFCreator is currently running.");

            COMLogger.Trace("COM: Starting pipe server thread");
              ThreadManager.Instance.StartPipeServerThread();
           
            COMLogger.Trace("COM: Starting initialization process");
            _isComActive = true;
            _comJobInfoQueue = JobInfoQueue.Instance;
            _comJobInfoQueue.OnNewJobInfo += (sender, eventArgs) => OnNewJob();
        }

        /// <summary>
        ///     Waits for exactly one job to enter the queue
        /// </summary>
        /// <param name="timeOut">Duration which the queue should wait for a job</param>
        /// <returns>False, if the duration was exceeded. Otherwise it returns true</returns>
        public bool WaitForJob(int timeOut)
        {
            return WaitForJobs(1, TimeSpan.FromSeconds(timeOut));
        }

        /// <summary>
        ///     Waits for n jobs to enter the queue
        /// </summary>
        /// <param name="jobCount">Number of jobs to wait for</param>
        /// <param name="timeOut">Duration which the queue should wait for the n jobs</param>
        /// <returns>False, if the duration was exceeded. Otherwise it returns true</returns>
        public bool WaitForJobs(int jobCount, int timeOut)
        {
            TimeSpan ts = TimeSpan.FromSeconds(timeOut);
            return WaitForJobs(jobCount, ts);
        }

        /// <summary>
        ///     Waits for n jobs to enter the queue
        /// </summary>
        /// <param name="jobCount">Number of jobs to wait for</param>
        /// <param name="timeOut">Duration which the queue should wait for a job</param>
        /// <returns>
        ///     False, if the duration was exceeded or the queue contains more or as many jobs as specified by jobCount.
        ///     Otherwise it returns true
        /// </returns>
        private bool WaitForJobs(int jobCount, TimeSpan timeOut)
        {
            if (!_isComActive)
                throw new COMException("No COM instance was found. Initialize the object first.");

            COMLogger.Trace("Waiting for {0} jobs for {1} seconds", jobCount, timeOut);

            DateTime maxTime = DateTime.Now + timeOut;

            while (_comJobInfoQueue.Count < jobCount)
            {
                if (DateTime.Now > maxTime)
                    return false;

                _newJobResetEvent.WaitOne(maxTime - DateTime.Now);
            }
            return true;
        }  

        /// <summary>
        ///     Returns the number of jobs in the queue
        /// </summary>
        public int Count
        {
            get
            {
                return _comJobInfoQueue.Count;
            }
        }

        /// <summary>
        ///     Calls the Set-Method of the AutoResetEvent when a new job enters the queue
        /// </summary>
        private void OnNewJob()
        {
            _newJobResetEvent.Set();
        }

        /// <summary>
        /// Returns the next job in the queue as a ComJob
        /// </summary>
        public PrintJob NextJob
        {
            get
            {
                return JobById(0);
            }
        }

        /// <summary>
        /// Creates the job from the queue by index
        /// </summary>
        /// <param name="jobIndex">Index of the jobinfo in the queue</param>
        /// <returns>The corresponding ComJob</returns>
        public PrintJob GetJobByIndex(int jobIndex)
        {
            return JobById(jobIndex);
        }

        /// <summary>
        /// Creates the job from the queue by index
        /// </summary>
        /// <param name="id">Index of the jobinfo in the queue</param>
        /// <returns>The corresponding ComJob</returns>
        private PrintJob JobById(int id)
        {
            try
            {
                IJobInfo currentJobInfo = _comJobInfoQueue.JobInfos[id];

                IJob currentJob = new GhostscriptJob(currentJobInfo, SettingsHelper.Settings.ApplicationSettings,
                    SettingsHelper.GetDefaultProfile());

                COMLogger.Trace("COM: Creating the ComJob from the queue determined by the index.");
                var indexedJob = new PrintJob(currentJob, _comJobInfoQueue);
                return indexedJob;
            }
            catch (ArgumentOutOfRangeException)
            {
                throw new COMException("Invalid index. Please check the index parameter.");
            }
        }

        /// <summary>
        /// Merges two ComJobs
        /// </summary>
        /// <param name="job1">The first job to merge</param>
        /// <param name="job2">The second job to merge</param>
        public void MergeJobs(PrintJob job1, PrintJob job2)
        {
            COMLogger.Trace("COM: Merging two ComJobs.");
            job1.JobInfo.Merge(job2.JobInfo);
            _comJobInfoQueue.Remove(job2.JobInfo);
        }

        /// <summary>
        /// Merges all jobs in the queue
        /// </summary>
        public void MergeAllJobs()
        {
            if(_comJobInfoQueue.Count == 0)
                throw new COMException("The queue must not be empty.");

            COMLogger.Trace("COM: Merging all ComJobs.");
            while (_comJobInfoQueue.Count > 1)
            {
                var firstJob = _comJobInfoQueue.JobInfos[0];
                var nextJob = _comJobInfoQueue.JobInfos[1];

                firstJob.Merge(nextJob);
                _comJobInfoQueue.Remove(nextJob);
            }
        }

        /// <summary>
        /// In case something went wrong, i.e. an exception was thrown, empties the whole queue.
        /// </summary>
        private void EmptyQueue()
        {
            for (var i = _comJobInfoQueue.JobInfos.Count-1; i >= 0; i--)
            {
                _comJobInfoQueue.Remove(_comJobInfoQueue.JobInfos[i], true);
            }
            _comJobInfoQueue = null;
        }

        /// <summary>
        ///  Shuts down the used instance
        /// </summary>
        public void ReleaseCom()
        {
            if (_isComActive)
            {
                if(!_threadPool.Join(4000))
                    throw new COMException("On of the thread jobs didn't finish within the time out.");

                COMLogger.Trace("COM: Cleaning up COM resources.");
                ThreadManager.Instance.PipeServer.Stop();

                _isServerInstanceRunning = false;
                _isComActive = false;

                COMLogger.Trace("COM: Emptying queue.");
                EmptyQueue();
            }
            else
            {
                throw new COMException("No COM Instance was found.");
            }
        }
    }
}