﻿using System;
using System.Collections.Generic;
using pdfforge.PDFCreator.Core.Settings;
using NLog.Config;
using NLog.Targets;
using System.IO;
using NLog;
using pdfforge.PDFCreator.Core.Settings.Enums;

namespace pdfforge.PDFCreator.Helper
{
    /// <summary>
    /// LoggingUtil provides functionality for setting up and managing the logging behaviour.
    /// </summary>
    internal class LoggingUtil
    {
        private static bool _initialized ;
        private static FileTarget _fileTarget;
        private static LoggingRule _loggingRule;
        private static string _logPath;

        private const string TraceLogLayout = "${shortdate} ${date:format=HH\\:mm\\:ss.ffff} [${level}] ${processid}-${threadid} (${threadname}) ${callsite}: ${message}";
        private const string ShortLogLayout = "${shortdate} ${date:format=HH\\:mm\\:ss.ffff} [${level}] ${callsite}: ${message}";

        public static string LogFile { get; private set; }

        /// <summary>
        /// Hiding constructor as there are only static methods
        /// </summary>
        private LoggingUtil()
        {

        }

        /// <summary>
        /// Initialize Logging, Targets and Rules
        /// </summary>
        public static void InitLogging(ApplicationSettings settings)
        {
            _initialized = true;

            var config = new LoggingConfiguration();
            LogManager.Configuration = config;

            _fileTarget = new FileTarget();
            _fileTarget.Layout = ShortLogLayout;
            _fileTarget.ArchiveAboveSize = 10485760; // 10MB
            _fileTarget.MaxArchiveFiles = 2;
            _logPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "PDFCreator");

            if (!Directory.Exists(_logPath))
                Directory.CreateDirectory(_logPath);

            LogFile = Path.Combine(_logPath, "PDFCreator.log");
            _fileTarget.FileName = LogFile;

            config.AddTarget("file", _fileTarget);

            ApplyLogging(settings.LoggingLevel);

            LogManager.Configuration = config;
        }

        /// <summary>
        /// Enable the file logging rule
        /// </summary>
        public static void EnableLogging(LogLevel logLevel)
        {
            if (!_initialized)
                throw new ArgumentException("Logging has not been initialized");

            if (!Directory.Exists(_logPath))
                Directory.CreateDirectory(_logPath);

            if (!LogManager.Configuration.LoggingRules.Contains(_loggingRule))
            {
                _loggingRule = new LoggingRule("*", logLevel, _fileTarget);
                LogManager.Configuration.LoggingRules.Add(_loggingRule);

                // Apply settings
                LogManager.Configuration = LogManager.Configuration;
            }
            else
            {
                ChangeLogLevel(logLevel);
            }
        }

        /// <summary>
        /// Disable all logging rules that use our file target
        /// </summary>
        public static void DisableLogging()
        {
            if (!_initialized)
                throw new ArgumentException("Logging has not been initialized");

            var rulesToDelete = new List<LoggingRule>();

            foreach (LoggingRule rule in LogManager.Configuration.LoggingRules)
            {
                if (rule.Targets.Contains(_fileTarget))
                {
                    rulesToDelete.Add(rule);
                }
            }

            foreach (LoggingRule rule in rulesToDelete)
            {
                LogManager.Configuration.LoggingRules.Remove(rule);
            }
            LogManager.Configuration = LogManager.Configuration;
        }

        /// <summary>
        /// Apply logging settings
        /// </summary>
        public static void ApplyLogging(LoggingLevel loggingLevel)
        {
            if (!_initialized)
                throw new ArgumentException("Logging has not been initialized");

            var logLevel = LogLevel.FromOrdinal((int)loggingLevel);

            _fileTarget.Layout = loggingLevel <= LoggingLevel.Trace ? TraceLogLayout : ShortLogLayout;

            if (logLevel != LogLevel.Off)
                EnableLogging(logLevel); 
            else
                DisableLogging();
        }

        private static void ChangeLogLevel(LogLevel logLevel)
        {
            if (!_initialized)
                throw new ArgumentException("Logging has not been initialized");

            LogManager.Configuration.LoggingRules.Remove(_loggingRule);

            _loggingRule = new LoggingRule("*", logLevel, _fileTarget);
            LogManager.Configuration.LoggingRules.Add(_loggingRule);

            LogManager.Configuration = LogManager.Configuration;
        }

        public static void ChangeLogLevel(LoggingLevel loggingLevel)
        {
            ChangeLogLevel(LogLevel.FromOrdinal((int)loggingLevel));
        }
    }
}
