﻿using System;
using System.Collections.Generic;
using pdfforge.DataStorage;

namespace pdfforge.PDFCreator.Helper
{
    /// <summary>
    /// The SettingsUpgrader class performs updates to the PDFCreator settings.
    /// This is done after loading the raw data and before loading them into the DataStorage classes.
    /// There is one update method from each version to the next (0 to 1, 1 to 2 etc.) and they are called subsequently, if required.
    /// </summary>
    public class SettingsUpgrader
    {
        private Data SettingsData { get; set; }

        private readonly List<Action<Data>> _upgradeMethods = new List<Action<Data>>();

        public const string VersionSettingPath = @"ApplicationProperties\SettingsVersion";

        public int SettingsVersion
        {
            get {
                string versionString = SettingsData.GetValue(VersionSettingPath);
                return GetInt(versionString) ?? 0;
            }
        }

        public SettingsUpgrader(Data settingsData)
        {
            SettingsData = settingsData;

            _upgradeMethods.Add(UpgradeV0ToV1);
            _upgradeMethods.Add(UpgradeV1ToV2);
        }

        public void Upgrade(int targetVersion)
        {
            for (int i = SettingsVersion; i < Math.Min(targetVersion, _upgradeMethods.Count); i++)
            {
                // Call upgrade methods subsequently, starting with the current version
                var upgradeMethod = _upgradeMethods[i];
                upgradeMethod(SettingsData);
            }
        }

        public bool RequiresUpgrade(int targetVersion)
        {
            return targetVersion > SettingsVersion;
        }

        private void UpgradeV0ToV1(Data data)
        {
            MoveSettingInAllProfiles(data, "DefaultFormat", "OutputFormat");
            MapSettingInAllProfiles(data, "PdfSettings\\Security\\" + "EncryptionLevel", MapEncryptionNamesV1);
            ApplyNewSettingInAllProfiles(data, "TitleTemplate", "<PrintJobName>");
            ApplyNewSettingInAllProfiles(data, "AuthorTemplate", "<PrintJobAuthor>");

            data.SetValue(VersionSettingPath, "1");
        }

        private void UpgradeV1ToV2(Data data)
        {
            MoveSettingInAllProfiles(data, @"CoverPage\AddBackground", @"BackgroundPage\OnCover");
            MoveSettingInAllProfiles(data, @"AttachmentPage\AddBackground", @"BackgroundPage\OnAttachment");
            MoveSetting(data, @"ApplicationSettings\LastUsedProfilGuid", @"ApplicationSettings\LastUsedProfileGuid");
            data.SetValue(VersionSettingPath, "2");
        }

        private string MapEncryptionNamesV1(string s)
        {
            switch (s)
            {
                case "Low40Bit":
                    return "Rc40Bit";
                case "Medium128Bit":
                    return "Rc128Bit";
                case "High128BitAes":
                    return "Aes128Bit";
            }

            return "Rc128Bit";
        }

        private void MoveSetting(Data data, string oldPath, string newPath)
        {
            var v = data.GetValue(oldPath);
            data.SetValue(newPath, v);
            data.RemoveValue(oldPath);
        }

        private void MoveSettingInAllProfiles(Data data, string oldPath, string newPath)
        {
            int? numProfiles = GetInt(data.GetValue(@"ConversionProfiles\numClasses"));

            if (numProfiles != null)
            {
                for (int i = 0; i < numProfiles; i++)
                {
                    string path = string.Format(@"ConversionProfiles\{0}\", i);
                    MoveSetting(data, path + oldPath, path + newPath);
                }
            }
        }

        private void MapSettingInAllProfiles(Data data, string path, Func<string,string> mapFunction)
        {
            int? numProfiles = GetInt(data.GetValue(@"ConversionProfiles\numClasses"));

            if (numProfiles != null)
            {
                for (int i = 0; i < numProfiles; i++)
                {
                    string p = string.Format(@"ConversionProfiles\{0}\" + path, i);
                    var v = data.GetValue(p);
                    data.SetValue(p, mapFunction(v));
                }
            }
        }

        private void ApplyNewSettingInAllProfiles(Data data, string path, string defaultValue)
        {
            int? numProfiles = GetInt(data.GetValue(@"ConversionProfiles\numClasses"));

            if (numProfiles != null)
            {
                for (int i = 0; i < numProfiles; i++)
                {
                    string p = string.Format(@"ConversionProfiles\{0}\" + path, i);
                    data.SetValue(p, defaultValue);
                }
            }
        }

        private int? GetInt(string s)
        {
            int i;

            if (!int.TryParse(s, out i))
                return null;
            return i;
        }
    }
}
