﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Windows.Data;
using pdfforge.DynamicTranslator;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Core.Settings.Enums;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.ViewModels.Wrapper;

namespace pdfforge.PDFCreator.ViewModels
{
    public class ApplicationSettingsViewModel : ViewModelBase
    {
        private ApplicationSettings _applicationSettings;
        private ApplicationProperties _applicationProperties;
        private ICollection<string> _pdfCreatorPrinters;
        private IList<Language> _languages;
        private SynchronizedCollection<TitleReplacement> _titleReplacements;
        private ICollectionView _titleReplacementView;
        private IEnumerable<ConversionProfile> _conversionProfiles;
        private SynchronizedCollection<PrinterMappingWrapper> _printerMappings;
        private ICollectionView _printerMappingView;

        private ConversionProfile dummyDefaultProfile = new ConversionProfile { Name = "<Last used profile>", Guid = "" };

        public ApplicationSettings ApplicationSettings  
        {
            get { return _applicationSettings; }
            set
            {
                _applicationSettings = value;
                ApplyTitleReplacements(value == null ? null : value.TitleReplacement);
                ApplyPrinterMappings();
                RaisePropertyChanged("ApplicationSettings");
            }
        }

        public ApplicationProperties ApplicationProperties
        {
            get { return _applicationProperties; }
            set
            {
                _applicationProperties = value;
                RaisePropertyChanged("ApplicationProperties");
            }
        }

        public ObservableCollection<PrinterMappingWrapper> PrinterMappings
        {
            get
            {
                if (_printerMappings == null)
                    return null;
                return _printerMappings.ObservableCollection;
            }
        }

        private void PrinterMappings_OnCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            _applicationSettings.PrinterMappings.Clear();

            foreach (var printerMappingWrapper in PrinterMappings)
            {
                _applicationSettings.PrinterMappings.Add(printerMappingWrapper.PrinterMapping);
            }

            RaiseTitleCommandsCanExecuteChanged();
        }

        private void PrinterMappingView_OnCurrentChanged(object sender, EventArgs eventArgs)
        {
            RenamePrinterCommand.RaiseCanExecuteChanged();
            DeletePrinterCommand.RaiseCanExecuteChanged();
        }

        public ObservableCollection<TitleReplacement> TitleReplacements
        {
            get
            {
                if (_titleReplacements == null)
                    return null;
                return _titleReplacements.ObservableCollection;
            }
        }

        public IEnumerable<ConversionProfile> ConversionProfiles    
        {
            get { return _conversionProfiles; }
            set { _conversionProfiles = value; RaisePropertyChanged("ConversionProfiles"); }
        }

        public IEnumerable<ConversionProfile> PrinterMappingProfiles
        {
            get
            {
                var profiles = _conversionProfiles.Where(x => !x.IsTemporary).ToList();
                dummyDefaultProfile.Name = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "LastUsedProfileMapping", "<Last used profile>");
                profiles.Insert(0, dummyDefaultProfile);
                return profiles;
            }
        }

        public ICollection<string> PdfCreatorPrinters
        {
            get { return _pdfCreatorPrinters; }
            set { _pdfCreatorPrinters = value; RaisePropertyChanged("PdfCreatorPrinters"); }
        }

        public IList<Language> Languages
        {
            get { return _languages; }
            set { _languages = value; RaisePropertyChanged("Languages"); }
        }

        public IEnumerable<AskSwitchPrinter> AskSwitchPrinterValues
        {
            get
            {
                return new List<AskSwitchPrinter>
                {
                    new AskSwitchPrinter(TranslationUtil.GetTranslation("ApplicationSettingsWindow", "Ask", "Ask"), true ),
                    new AskSwitchPrinter(TranslationUtil.GetTranslation("ApplicationSettingsWindow", "Yes", "Yes"), false )
                };
            }
        }

        public IEnumerable<EnumValue<UpdateInterval>> UpdateIntervals
        {
            get { return TranslationUtil.GetEnumTranslation<UpdateInterval>(); }
        }

        public IEnumerable<EnumValue<LoggingLevel>> LoggingValues
        {
            get { return TranslationUtil.GetEnumTranslation<LoggingLevel>(); }
        }

        public Action AddPrinterAction { private get; set; }
        public Action<PrinterMappingWrapper> RenamePrinterAction { private get; set; }
        public Action<PrinterMappingWrapper> DeletePrinterAction { private get; set; }

        public Func<ICollection<string>> GetPrinterListAction { get; set; }

        public DelegateCommand AddPrinterCommand { get; private set; }
        public DelegateCommand RenamePrinterCommand { get; private set; }
        public DelegateCommand DeletePrinterCommand { get; private set; }

        public DelegateCommand TitleMoveUpCommand { get; set; }
        public DelegateCommand TitleMoveDownCommand { get; set; }
        public DelegateCommand TitleAddCommand { get; set; }
        public DelegateCommand TitleDeleteCommand { get; set; }

        public ApplicationSettingsViewModel(ApplicationSettings applicationSettings, ApplicationProperties applicationProperties, IEnumerable<ConversionProfile> profiles, Func<ICollection<string>> fetchPrintersFunc, IList<Language> languages)
        {
            ApplicationSettings = applicationSettings;
            ApplicationProperties = applicationProperties;
            GetPrinterListAction = fetchPrintersFunc;
            Languages = languages;

            AddPrinterCommand = new DelegateCommand(AddPrintercommandExecute);
            RenamePrinterCommand = new DelegateCommand(RenamePrinterCommandExecute, ModifyPrinterCommandCanExecute);
            DeletePrinterCommand = new DelegateCommand(DeletePrinterCommandExecute, ModifyPrinterCommandCanExecute);

            TitleMoveUpCommand = new DelegateCommand(TitleMoveUpCommandExecute, TitleMoveUpCommandCanExecute);
            TitleMoveDownCommand = new DelegateCommand(TitleMoveDownCommandExecute, TitleMoveDownCommandCanExecute);
            TitleAddCommand = new DelegateCommand(TitleAddCommandExecute);
            TitleDeleteCommand = new DelegateCommand(TitleDeleteCommandExecute, TitleDeleteCommandCanExecute);

            ConversionProfiles = profiles;

            UpdatePrinterList();
        }

        public ApplicationSettingsViewModel()
            : this(null, null, new List<ConversionProfile>(), PrinterUtil.GetPDFCreatorPrinters, Translator.FindTranslations(TranslationUtil.TranslationPath))
        {
        }

        public void UpdatePrinterList()
        {
            if (GetPrinterListAction == null)
                return;

            if (_pdfCreatorPrinters == null)
                _pdfCreatorPrinters = new List<string>();

            _pdfCreatorPrinters.Clear();
            foreach (var printer in GetPrinterListAction())
            {
                _pdfCreatorPrinters.Add(printer);
            }
        }

        public void UpdatePrimaryPrinter(string printerName)
        {
            foreach (var printerMappingWrapper in _printerMappings.ObservableCollection)
            {
                printerMappingWrapper.PrimaryPrinter = printerName;
            }
        }

        private void ApplyPrinterMappings()
        {
            if (ApplicationSettings != null && ApplicationSettings.PrinterMappings != null)
            {
                var mappingWrappers = new List<PrinterMappingWrapper>();
                
                foreach (var printerMapping in ApplicationSettings.PrinterMappings)
                {
                    var mappingWrapper = new PrinterMappingWrapper(printerMapping);
                    if (mappingWrapper.Profile == null)
                    {
                        mappingWrapper.Profile = dummyDefaultProfile;
                    }
                    mappingWrappers.Add(mappingWrapper);
                }

                _printerMappings = new SynchronizedCollection<PrinterMappingWrapper>(mappingWrappers);

                _printerMappings.ObservableCollection.CollectionChanged += PrinterMappings_OnCollectionChanged;
                _printerMappingView = CollectionViewSource.GetDefaultView(_printerMappings.ObservableCollection);
                _printerMappingView.CurrentChanged += PrinterMappingView_OnCurrentChanged;
            }

            RaisePropertyChanged("PrinterMappings");
        }

        private void ApplyTitleReplacements(IEnumerable<TitleReplacement> titleReplacements)
        {
            if (TitleReplacements != null)
            {
                TitleReplacements.CollectionChanged -= TitleReplacementsOnCollectionChanged;
            }

            if (titleReplacements != null)
            {
                var replacements = titleReplacements as IList<TitleReplacement> ?? titleReplacements.ToList();

                _titleReplacements = new SynchronizedCollection<TitleReplacement>(replacements);
                _titleReplacements.ObservableCollection.CollectionChanged += TitleReplacementsOnCollectionChanged;
                _titleReplacementView = CollectionViewSource.GetDefaultView(_titleReplacements.ObservableCollection);
                _titleReplacementView.CurrentChanged += TitleReplacement_OnCurrentChanged;
            }

            RaisePropertyChanged("TitleReplacements");
        }

        private void TitleReplacement_OnCurrentChanged(object sender, EventArgs eventArgs)
        {
            RaiseTitleCommandsCanExecuteChanged();
        }

        private void RaiseTitleCommandsCanExecuteChanged()
        {
            TitleMoveUpCommand.RaiseCanExecuteChanged();
            TitleMoveDownCommand.RaiseCanExecuteChanged();
            TitleDeleteCommand.RaiseCanExecuteChanged();
        }

        private bool TitleMoveUpCommandCanExecute(object obj)
        {
            if (_titleReplacementView == null || _titleReplacementView.CurrentItem == null)
                return false;

            return _titleReplacements.ObservableCollection.IndexOf(_titleReplacementView.CurrentItem as TitleReplacement) > 0;
        }

        private void TitleMoveUpCommandExecute(object obj)
        {
            int index = TitleReplacements.IndexOf(_titleReplacementView.CurrentItem as TitleReplacement);
            TitleReplacements.Move(index, index - 1);
        }

        private bool TitleMoveDownCommandCanExecute(object obj)
        {
            if (_titleReplacementView == null || _titleReplacementView.CurrentItem == null)
                return false;

            return _titleReplacements.ObservableCollection.IndexOf(_titleReplacementView.CurrentItem as TitleReplacement) < (_titleReplacements.ObservableCollection.Count - 1);
        }

        private void TitleMoveDownCommandExecute(object obj)
        {
            int index = TitleReplacements.IndexOf(_titleReplacementView.CurrentItem as TitleReplacement);
            TitleReplacements.Move(index, index + 1);
        }

        private bool TitleDeleteCommandCanExecute(object obj)
        {
            if (_titleReplacementView == null || _titleReplacementView.CurrentItem == null || _titleReplacementView.CurrentItem == CollectionView.NewItemPlaceholder)
                return false;

            return true;
        }

        private void TitleDeleteCommandExecute(object obj)
        {
            TitleReplacements.Remove(_titleReplacementView.CurrentItem as TitleReplacement);
        }

        private void TitleAddCommandExecute(object obj)
        {
            var newItem = new TitleReplacement();
            TitleReplacements.Add(newItem);
            _titleReplacementView.MoveCurrentTo(newItem);
        }

        private void TitleReplacementsOnCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            RaiseTitleCommandsCanExecuteChanged();
        }

        private bool ModifyPrinterCommandCanExecute(object o)
        {
            if (_printerMappingView == null)
                return false;

            var currentMapping = _printerMappingView.CurrentItem as PrinterMappingWrapper;

            if (currentMapping == null)
                return false;

            return PdfCreatorPrinters.Contains(currentMapping.PrinterName);
        }

        private void AddPrintercommandExecute(object o)
        {
            AddPrinterAction();
            ApplyPrinterMappings(); //Required to replace empty guid with dummyDefaultProfile (in english <Last used profile>)
            UpdatePrinterCollectionViews();
        }

        private void RenamePrinterCommandExecute(object obj)
        {
            var currentMapping = _printerMappingView.CurrentItem as PrinterMappingWrapper;

            if (currentMapping == null)
                return;

            RenamePrinterAction(currentMapping);
        }

        private void DeletePrinterCommandExecute(object obj)
        {
            var currentMapping = _printerMappingView.CurrentItem as PrinterMappingWrapper;

            if (currentMapping == null)
                return;

            DeletePrinterAction(currentMapping);
        }

        private void UpdatePrinterCollectionViews()
        {
            UpdatePrinterList();
            CollectionViewSource.GetDefaultView(ApplicationSettings.PrinterMappings).Refresh();
            CollectionViewSource.GetDefaultView(PdfCreatorPrinters).Refresh();
        }
    }

    public class AskSwitchPrinter
    {
        public bool Value { get; set; }
        public string Name { get; set; }

        public AskSwitchPrinter(string name, bool value)
        {
            Value = value;
            Name = name;
        }
    }
}
