﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Navigation;
using pdfforge.DataStorage.Storage;
using pdfforge.PDFCreator.Core;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.ViewModels;
using pdfforge.PDFCreator.ViewModels.Wrapper;
using PrinterUtil = pdfforge.PDFCreator.Helper.PrinterUtil;

namespace pdfforge.PDFCreator.Views
{
    public partial class ApplicationSettingsWindow : Window
    {
        public Visibility RequiresUacVisibility
        {
            get { return OsHelper.UserIsAdministrator() ? Visibility.Collapsed : Visibility.Visible; }
        }
        
        private ApplicationSettingsViewModel ViewModel
        {
            get { return (ApplicationSettingsViewModel) DataContext; }
        }

        public ApplicationSettingsWindow(ApplicationSettings applicationSettings, ApplicationProperties applicationProperties, IEnumerable<ConversionProfile> conversionProfiles )
            : this()
        {
            ViewModel.ApplicationSettings = applicationSettings;
            ViewModel.ConversionProfiles = SettingsHelper.Settings.ConversionProfiles;
            ViewModel.ApplicationProperties = applicationProperties;
            ViewModel.ConversionProfiles = conversionProfiles;

            ViewModel.AddPrinterAction = AddPrinterAction;
            ViewModel.RenamePrinterAction = RenamePrinterAction;
            ViewModel.DeletePrinterAction = DeletePrinterAction;

            if (PdfArchitectHelper.IsPdfArchitectInstalled)
            {
                LaunchPdfArchitectStackPanel.Visibility = Visibility.Visible;
                GetPdfArchitectStackPanel.Visibility = Visibility.Collapsed;
            }
            else
            {
                LaunchPdfArchitectStackPanel.Visibility = Visibility.Collapsed;
                GetPdfArchitectStackPanel.Visibility = Visibility.Visible;
            }
        }

        private void AddPrinterAction()
        {
            string printerName = CreateValidPrinterName("PDFCreator");
            string questionText = TranslationUtil.GetTranslation("InputBoxWindow", "EnterPrintername",
                "Please enter printer name:");
            printerName = RequestPrinternameFromUser(questionText, printerName);
            if (printerName == null)
                return;

            while (!PrinterUtil.IsValidPrinterName(printerName))
            {
                questionText = TranslationUtil.GetFormattedTranslation("ApplicationSettingsWindow", "PrinterAlreadyInstalled",
                    "A printer with the name '{0}' is already installed on your system. Please enter a new printer name:",
                    printerName);
                printerName = CreateValidPrinterName(printerName);
                printerName = RequestPrinternameFromUser(questionText, printerName);
                if (printerName == null)
                    return;
            }

            var ret = CallPrinterHelper("/AddPrinter " + printerName);

            if (ret)
            {
                ViewModel.PrinterMappings.Add(new PrinterMappingWrapper(new PrinterMapping(printerName, SettingsHelper.LAST_USED_PROFILE_GUID)));
            }
        }

        private void RenamePrinterAction(PrinterMappingWrapper printerMapping)
        {
            string oldPrinterName = printerMapping.PrinterName;

            if (oldPrinterName.Length == 0)
                return;

            string questionText = TranslationUtil.GetTranslation("InputBoxWindow", "EnterPrintername",
                "Please enter printer name:");
            string newPrinterName = RequestPrinternameFromUser(questionText, oldPrinterName);
            if ((newPrinterName == null) || (newPrinterName == oldPrinterName))
                return;

            while (!PrinterUtil.IsValidPrinterName(newPrinterName))
            {
                questionText = TranslationUtil.GetFormattedTranslation("ApplicationSettingsWindow", "PrinterAlreadyInstalled",
                    "A printer with the name '{0}' is already installed on your system. Please enter a new printer name:",
                    newPrinterName);
                newPrinterName = CreateValidPrinterName(newPrinterName);
                newPrinterName = RequestPrinternameFromUser(questionText, newPrinterName);
                if ((newPrinterName == null) || (newPrinterName == oldPrinterName))
                    return;
            }


            if (CallPrinterHelper("/RenamePrinter " + oldPrinterName + " " + newPrinterName))
            {
                ViewModel.PdfCreatorPrinters = PrinterUtil.GetPDFCreatorPrinters();
                printerMapping.PrinterName = newPrinterName;

                if (PrimaryPrinterBox.SelectedValue.ToString() == oldPrinterName)
                    PrimaryPrinterBox.SelectedValue = newPrinterName;
            }
        }

        private void DeletePrinterAction(PrinterMappingWrapper printerMapping)
        {
            var printerName = printerMapping.PrinterName;

            if (ViewModel.PdfCreatorPrinters.Count < 2)
            {
                var message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "DontDeleteLastPrinter",
                    "You may not delete the last printer. Uninstall PDFCreator if you really want to remove all related printers.");
                const string caption = @"PDFCreator";
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                return;
            }

            var msg = TranslationUtil.GetFormattedTranslation("ApplicationSettingsWindow", "AskDeletePrinter",
                "Are you sure that you want to delete the printer '{0}'?", printerName);
            var cpt = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "DeletePrinter", "Delete Printer");
            if (MessageWindow.ShowTopMost(msg, cpt, MessageWindowButtons.YesNo, MessageWindowIcon.Question) != MessageWindowResponse.Yes)
                return;

            if (CallPrinterHelper("/DeletePrinter " + printerName))
            {
                ViewModel.PrinterMappings.Remove(printerMapping);

                if (PrimaryPrinterBox.SelectedValue.ToString() == printerName)
                    PrimaryPrinterBox.SelectedValue = PrinterUtil.GetApplicablePDFCreatorPrinter("PDFCreator", "PDFCreator");
            }
        }

        public ApplicationSettingsWindow()
        {
            InitializeComponent();
        }

        private string CreateValidPrinterName(string baseName)
        {
            int i = 2;
            string printerName = baseName;

            while (!PrinterUtil.IsValidPrinterName(printerName))
            {
                printerName = baseName + i++;
            }

            return printerName;
        }

        private string RequestPrinternameFromUser(string questionText, string printerName)
        {
            var w = new InputBoxWindow();
            w.IsValidInput = ValidatePrinterName;
            w.QuestionText = questionText;
            w.InputText = printerName;

            if (w.ShowDialog() != true)
                return null;

            return w.InputText;
        }

        private InputBoxValidation ValidatePrinterName(string arg)
        {
            if (PrinterUtil.IsValidPrinterName(arg))
                return new InputBoxValidation(true, "");

            return new InputBoxValidation(false, TranslationUtil.GetTranslation("ApplicationSettingsWindow", "InvalidPrinterName",
                    "The name is invalid or a printer with this name already exists"));
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        private void LanguagePreviewButton_Click(object sender, RoutedEventArgs e)
        {
            TranslationUtil.SetTemporaryTranslation((DynamicTranslator.Language)LanguageBox.SelectionBoxItem);
            TranslationUtil.TranslatorInstance.Translate(this);
            TranslationUtil.TranslateProfileList(SettingsHelper.Settings.ConversionProfiles);
            //overwrite items of comboboxes with translated items  
            UpdateIntervalComboBox.ItemsSource = ViewModel.UpdateIntervals;
            UpdateIntervalComboBox.SelectedValue = ViewModel.ApplicationSettings.UpdateInterval;
            ChangeDefaultPrinterComboBox.ItemsSource = ViewModel.AskSwitchPrinterValues;
            ChangeDefaultPrinterComboBox.SelectedValue = ViewModel.ApplicationSettings.AskSwitchDefaultPrinter;
            TranslationUtil.RevertTemporaryTranslation();
        }

        private void DownloadHyperlink_OnRequestNavigate(object sender, RequestNavigateEventArgs e)
        {
            Process.Start(Urls.PdfforgeWebsiteUrl);
            e.Handled = true;
        }

        private void UpdateCheckButton_OnClick(object sender, RoutedEventArgs e)
        {
            if (!UpdateManager.Instance.UpdateProcedureIsRunning)
            {
                UpdateManager.Instance.Initialize(ViewModel.ApplicationSettings, ViewModel.ApplicationProperties);
                UpdateManager.Instance.NewVersionEvent += UpdateManager.LaunchNewUpdateForm;
                UpdateManager.Instance.NoNewVersionEvent += UpdateManager.LaunchNoUpdateForm;
                UpdateManager.Instance.ErrorEvent += UpdateManager.LaunchErrorForm;
                UpdateManager.Instance.FinishedEvent += StoreNextUpdateAndEnableBtn;
                UpdateManager.Instance.UpdateProcedure(false);
            }
            else
            {
                var message = TranslationUtil.GetTranslation("UpdateManager",
                    "UpdateCheckIsRunning", "Update-Check is already running.");
                var caption = TranslationUtil.GetTranslation("UpdateManager",
                    "PDFCreatorUpdate", "PDFCreator Update");
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Info);
            }
        }

        private void StoreNextUpdateAndEnableBtn(object sender, UpdateEventArgs eventArgs)
        {
            Dispatcher.BeginInvoke(
                new ThreadStart(() =>
                {
                    ViewModel.ApplicationProperties.NextUpdate = UpdateManager.NextUpdate;
                }));
        }

        private void PrimaryPrinterBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (ViewModel == null)
                return;

            ViewModel.UpdatePrimaryPrinter(ViewModel.ApplicationSettings.PrimaryPrinter);
        }

        /// <summary>
        /// Call the PrinterHelper.exe to add, rename or delete printers
        /// </summary>
        /// <param name="arguments">Command line arguments that will be passed to PrinterHelper.exe</param>
        /// <returns>true, if the action was successful</returns>
        private bool CallPrinterHelper(string arguments)
        {
            string assemblyPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            if (assemblyPath == null)
                return false;

            string printerHelperPath = Path.Combine(assemblyPath, "PrinterHelper.exe");

            if (!File.Exists(printerHelperPath))
            {
                var message = TranslationUtil.GetFormattedTranslation("Application", "SetupFileMissing",
                    "An important PDFCreator file is missing ('{0}'). Please reinstall PDFCreator!",
                    Path.GetFileName(printerHelperPath));
                var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "Error", "Error");
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                return false;
            }

            if (Environment.OSVersion.Version.Major <= 5)
            {
                if (!OsHelper.UserIsAdministrator())
                {
                    var message = TranslationUtil.GetFormattedTranslation("ApplicationSettingsWindow", "OperationRequiresAdminPrivileges",
                    "This operation requires admin privileges and it looks like you are not an admin. Do you want to continue anyway?\r\nNote: It is safe to continue even if you are unsure if you have approrpiate rights, but the operation will not be completed.",
                    Path.GetFileName(printerHelperPath));
                    var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "AdminPrivilegesRequired", "Admin privileges required");
                    if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Info) ==
                        MessageWindowResponse.No)
                        return false;
                }
            }

            return CallProgramAsAdmin(printerHelperPath, arguments);
        }

        private void ShowLogButton_OnClick(object sender, RoutedEventArgs e)
        {
            if (File.Exists(LoggingUtil.LogFile))
            {
                Process.Start(LoggingUtil.LogFile);
            }
            else
            {
                string caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "NoLogFile", "No log file");
                string message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "NoLogFileAvailable", "There is currently no log file available.");
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Warning);
            }
        }

        private void ClearLogButton_OnClick(object sender, RoutedEventArgs e)
        {
            if (File.Exists(LoggingUtil.LogFile))
            {
                File.WriteAllText(LoggingUtil.LogFile, "");
            }
        }

        private void PdfCreatorTestPageButton_OnClick(object sender, RoutedEventArgs e)
        {
            if (!QuerySaveModifiedSettings())
                return;

            JobInfoQueue.Instance.AddTestPage();
        }

        private void WindowsTestPageButton_OnClick(object sender, RoutedEventArgs e)
        {
            if (!QuerySaveModifiedSettings())
                return;

                PrinterUtil.PrintWindowsTestPage(ViewModel.ApplicationSettings.PrimaryPrinter);
        }

        private bool AppSettingsAreModified()
        {
            return !ViewModel.ApplicationSettings.Equals(SettingsHelper.Settings.ApplicationSettings);
        }

        private void SaveAppSettings()
        {
            SettingsHelper.Settings.ApplicationSettings = ViewModel.ApplicationSettings.Copy();
            SettingsHelper.ApplySettings(SettingsHelper.Settings);
        }

        private bool QuerySaveModifiedSettings()
        {
            if (!AppSettingsAreModified())
                return true; //No changes -> proceed

            string message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "AskSaveModifiedSettings",
                "You have modified the application settings. In order to print a test page with the new settings, you have to save them first.\r\nDo you want to save the application settings now?");
            string caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "AppSettings", "Application Settings");

            var response = MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Question);
            if (response == MessageWindowResponse.Yes) //Proceed with saved settings
            {
                SaveAppSettings();
                return true;
            }
            if (response == MessageWindowResponse.No) //Proceed with old settings
            {
                return true;
            }
            return false; //Cancel Testprinting
        }

        private void ShowConextBasedHelp()
        {
            var active = TabControl.SelectedItem as TabItem;
            if (active == GeneralTab)
                HelpFileUtil.ShowHelp(HelpTopic.AppGeneral);
            else if (active == PrinterTab)
                HelpFileUtil.ShowHelp(HelpTopic.AppPrinters);
            else if (active == TitleTab)
                HelpFileUtil.ShowHelp(HelpTopic.AppTitle);
            else if (active == ApiTab)
                HelpFileUtil.ShowHelp(HelpTopic.AppApiServices);
            else if (active == DebugTab)
                HelpFileUtil.ShowHelp(HelpTopic.AppDebug);
            else
                HelpFileUtil.ShowHelp(HelpTopic.AppGeneral);
        }

        private void HelpButton_OnClick(object sender, RoutedEventArgs e)
        {
            ShowConextBasedHelp();
        }

        private void ApplicationSettingsWindow_OnKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.F1)
                ShowConextBasedHelp();
        }

        private void SaveIniButton_OnClick(object sender, RoutedEventArgs e)
        {
            var saveFileDialog = new Microsoft.Win32.SaveFileDialog();

            saveFileDialog.Filter = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "IniFileFilter", "INI files (*.ini)|*.ini");
            saveFileDialog.FileName = "PDFCreator.ini";

            if (saveFileDialog.ShowDialog() != true)
                return;

            var ini = new IniStorage(Encoding.UTF8);
            SettingsHelper.Settings.SaveData(ini, saveFileDialog.FileName);
        }

        private void LoadIniButton_OnClick(object sender, RoutedEventArgs e)
        {
            var openFileDialog = new Microsoft.Win32.OpenFileDialog();

            openFileDialog.Filter = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "IniFileFilter", "INI files (*.ini)|*.ini");
            openFileDialog.FileName = "PDFCreator.ini";

            if (openFileDialog.ShowDialog() != true)
                return;

            var message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "LoadSettingsFromFileWarning",
                "By loading the file all settings and profiles will be overwritten. Do you want to continue?");
            var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "OverwriteAllSettings",
                "Overwrite all settings");
            if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Warning) != MessageWindowResponse.Yes)
                return;

            var ini = new IniStorage(Encoding.UTF8);

            PdfCreatorSettings settings = SettingsHelper.CreateEmptySettings();
            settings.LoadData(ini, openFileDialog.FileName, SettingsHelper.UpgradeSettings);

            if (!SettingsHelper.CheckValidSettings(settings))
            {
                message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "InvalidSettingsWarning",
                    "The file does not appear to contain valid PDFCreator settings.");
                caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "InvalidSettings", "Invalid settings");

                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                return;
            }

            SettingsHelper.ApplySettings(settings);

            var viewModel = ViewModel;
            DataContext = null;
            viewModel.ApplicationSettings = settings.ApplicationSettings;
            DataContext = viewModel;
        }

        private void ApplicationSettingsWindow_OnLoaded(object sender, RoutedEventArgs e)
        {
            TranslationUtil.TranslatorInstance.Translate(this);
        }

        private void DataGrid_OnAutoGeneratingColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {           
            e.Cancel = true;
        }

        private bool CallProgramAsAdmin(string path, string arguments)
        {
            var result = ShellExecuteHelper.RunAsAdmin(path, arguments);

            if (result == ShellExecuteResult.RunAsWasDenied)
            {
                var message = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "SufficientPermissions",
                    "Operation failed. Probably You do not have sufficient permissions.");
                var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "Error", "Error");

                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);

                return false;
            }
            
            return result == ShellExecuteResult.Success;

        }

        /// <summary>
        ///     Call the SetupHelper.exe to add or remove explorer conext menu integration
        /// </summary>
        /// <param name="arguments">Command line arguments that will be passed to SetupHelper.exe</param>
        /// <returns>true, if the action was successful</returns>
        private bool CallSetupHelper(string arguments)
        {
            string assemblyPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            if (assemblyPath == null)
                return false;

            string setupHelperPath = Path.Combine(assemblyPath, "SetupHelper.exe");

            if (!File.Exists(setupHelperPath))
            {
                var message = TranslationUtil.GetFormattedTranslation("Application", "SetupFileMissing",
                    "An important PDFCreator file is missing ('{0}'). Please reinstall PDFCreator!",
                    Path.GetFileName(setupHelperPath));
                var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "Error", "Error");
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                return false;
            }

            if (Environment.OSVersion.Version.Major <= 5)
            {
                if (!OsHelper.UserIsAdministrator())
                {
                    var message = TranslationUtil.GetFormattedTranslation("ApplicationSettingsWindow", "OperationRequiresAdminPrivileges",
                    "This operation requires admin privileges and it looks like you are not an admin. Do you want to continue anyway?\r\nNote: It is safe to continue even if you are unsure if you have approrpiate rights, but the operation will not be completed.",
                    Path.GetFileName(setupHelperPath));
                    var caption = TranslationUtil.GetTranslation("ApplicationSettingsWindow", "AdminPrivilegesRequired", "Admin privileges required");
                    if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Info) == MessageWindowResponse.No)
                        return false;
                }
            }

            return CallProgramAsAdmin(setupHelperPath, arguments);
        }

        private void RemoveExplorerIntegrationButton_OnClick(object sender, RoutedEventArgs e)
        {
            CallSetupHelper("/FileExtensions=Remove");
        }

        private void AddExplorerIntegrationButton_OnClick(object sender, RoutedEventArgs e)
        {
            CallSetupHelper("/FileExtensions=Add /Description=\"" +
                            TranslationUtil.GetTranslation("Program", "ExplorerIntegration",
                                "Create PDF and image files with PDFCreator") + "\"");
        }

        private void LaunchPdfArchitectButton_OnClick(object sender, RoutedEventArgs e)
        {
            string applicationPath = PdfArchitectHelper.InstallPath;

            if (!string.IsNullOrWhiteSpace(applicationPath))
                Process.Start(applicationPath);
        }

        private void GetPdfArchitectButton_OnClick(object sender, RoutedEventArgs e)
        {
            Process.Start(Urls.ArchitectWebsiteUrl);
        }

        private void DownloadPdfArchitectButton_OnClickPdfArchitectButton_OnClick(object sender, RoutedEventArgs e)
        {
            Process.Start(Urls.ArchitectDownloadUrl);
        }
    }
}

