﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using pdfforge.PDFCreator.Converter;
using pdfforge.PDFCreator.Core.Actions;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.Utilities.Tokens;
using pdfforge.PDFCreator.ViewModels;
using pdfforge.PDFCreator.Views.ActionControls;
using ComboBox = System.Windows.Controls.ComboBox;
using KeyEventArgs = System.Windows.Input.KeyEventArgs;
using OpenFileDialog = Microsoft.Win32.OpenFileDialog;
using TextBox = System.Windows.Controls.TextBox;

namespace pdfforge.PDFCreator.Views
{
    public partial class ProfileSettingsWindow : Window
    {
        private readonly TokenReplacer _tokenReplacer = TokenHelper.TokenReplacerWithPlaceHolders;
        private ObservableCollection<ActionBundle> _actions = new ObservableCollection<ActionBundle>();

        private ProfileSettingsViewModel ViewModel
        {
            get { return (ProfileSettingsViewModel)DataContext; }
            set { DataContext = value; }
        }

        public PdfCreatorSettings Settings
        {
            get
            {
                return ViewModel.Settings;
            }
        }

        public ObservableCollection<ActionBundle> Actions
        {
            get { return _actions; }
            private set { _actions = value; }
        }

        private ProfileSettingsViewModel CreateViewModel(PdfCreatorSettings settings)
        {
            var viewModel = new ProfileSettingsViewModel(settings);

            viewModel.QueryProfileName = QueryProfileName;
            viewModel.QueryDeleteProfile = QueryDeleteProfile;
            viewModel.QueryDeleteProfileWithPrinterMapping = QueryDeleteProfileWithPrinterMapping;
            viewModel.QueryDiscardChanges = QueryDiscardUnsavedChanges;
            viewModel.QueryDiscardChangesInOtherProfiles = QueryDiscardChangesInOtherProfiles;
            viewModel.QueryIgnoreDefectiveProfiles = QueryIgnoreDefectiveProfiles;
            viewModel.UpdateLayoutProfilesBoxAction = ProfilesBoxUpdateLayout;

            viewModel.CloseViewAction = delegate(bool? result) { DialogResult = result; };

            viewModel.InvalidProfileMessage = TranslationUtil.GetTranslation("ProfileSettingsWindow", "InvalidProfileName", "The entered profile name is invalid or does already exist.");

            return viewModel;
        }

        private TokenReplacerConverter TokenReplacerConverter
        {
            get { return FindResource("TokenReplacerConverter") as TokenReplacerConverter; }
        }

        public ProfileSettingsWindow(PdfCreatorSettings settings)
        {
            InitializeComponent();
            ViewModel = CreateViewModel(settings);

            AddAction(new OpenViewerActionControl());
            AddAction(new BackgroundActionControl());
            AddAction(new CoverActionControl());
            AddAction(new AttachmentActionControl());
            AddAction(new PrintActionControl());
            AddAction(new EmailClientActionControl());
            AddAction(new EmailSmtpActionControl());
            AddAction(new ScriptActionControl());
            AddAction(new FtpActionControl(_tokenReplacer));
            //AddAction(new DummyControl("AttachMe", "AttachMeActionSettings"));

            foreach (var actionBundle in Actions)
            {
                actionBundle.UserControl.DataContext = DataContext;
            }

            foreach (string token in TokenHelper.GetTokenListForTitle())
                TitleTokensComboBox.Items.Add(token);

            foreach (string token in TokenHelper.GetTokenListForAuthor())
                AuthorTokensComboBox.Items.Add(token);

            foreach (string token in TokenHelper.GetTokenListForFilename())
                FilenameTokensComboBox.Items.Add(token);

            foreach (string token in TokenHelper.GetTokenListForDirectory())
            {
                SaveDialogFolderTokensComboBox.Items.Add(token);
                TargetFolderTokensComboBox.Items.Add(token);
            }

            ViewModel.CurrentProfilePropertyChanged();

            ViewModel.ConversionProfilesView.CurrentChanged += OnConversionProfilesViewOnCurrentChanged;

            if (ViewModel.CurrentProfile != null)
                UpdateFontLabel(ViewModel.CurrentProfile.Stamping);

            TokenReplacerConverter.TokenReplacer = TokenHelper.TokenReplacerWithPlaceHolders;
        }

        private void AddAction(IActionControl action)
        {
            var actionBundle = new ActionBundle(action);
            actionBundle.UserControl.DataContext = DataContext;
            Actions.Add(actionBundle);
        }

        private void SelectFirstEnabledOrFirstAction()
        {
            if (Actions == null)
                return;

            foreach (var action in Actions)
            {
                if (action.IsEnabled)
                {
                    ActionListBox.SelectedItem = action;
                    return;
                }
            }

            ActionListBox.SelectedIndex = 0;

        }

        private void OnConversionProfilesViewOnCurrentChanged(object sender, EventArgs e)
        {
            foreach (var actionBundle in Actions)
            {
                actionBundle.RaiseIsEnabledChanged();
            }

            if (ViewModel.CurrentProfile != null)
                UpdateFontLabel(ViewModel.CurrentProfile.Stamping);
        }

        private void ProfileSettingsWindow_OnKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.F1)
                ShowConextBasedHelp();
        }

        private void ShowConextBasedHelp()
        {
            var helpTopic = HelpTopic.General;

            var active = TabControl.SelectedItem as TabItem;
            if (active == DocumentTab)
                helpTopic = HelpTopic.ProfileDocument;
            else if (active == SaveTab)
                helpTopic = HelpTopic.ProfileSave;
            else if (active == ImageFormatsTab)
                helpTopic = HelpTopic.ProfileImageFormats;
            else if (active == PdfTab)
            {
                helpTopic = HelpTopic.ProfilePdf;

                if (PdfSettingsTabControl.SelectedItem == PdfGeneralTab)
                    helpTopic = HelpTopic.PdfGeneral;
                else if (PdfSettingsTabControl.SelectedItem == PdfSecurityTab)
                    helpTopic = HelpTopic.PdfSecurity;
                else if (PdfSettingsTabControl.SelectedItem == PdfSignatureTab)
                    helpTopic = HelpTopic.PdfSignature;
                else if (PdfSettingsTabControl.SelectedItem == PdfCompressionTab)
                    helpTopic = HelpTopic.PdfCompression;
            }
            else if (active == ActionsTab)
            {
                helpTopic = HelpTopic.ProfileActions;

                var currentAction = CollectionViewSource.GetDefaultView(Actions).CurrentItem as ActionBundle;
                if (currentAction != null)
                {
                    var currentActionControl = currentAction.UserControl;
                    if (currentActionControl is CoverActionControl)
                        helpTopic = HelpTopic.Cover;
                    else if (currentActionControl is AttachmentActionControl)
                        helpTopic = HelpTopic.Attachment;
                    else if (currentActionControl is EmailClientActionControl)
                        helpTopic = HelpTopic.OpenEmailClient;
                    else if (currentActionControl is EmailSmtpActionControl)
                        helpTopic = HelpTopic.SendEmailOverSmtp;
                    else if (currentActionControl is FtpActionControl)
                        helpTopic = HelpTopic.UploadWithFtp;
                    else if (currentActionControl is PrintActionControl)
                        helpTopic = HelpTopic.PrintDocument;
                    else if (currentActionControl is ScriptActionControl)
                        helpTopic = HelpTopic.RunScript;
                }
            }
            else
                helpTopic = HelpTopic.ProfileSettings;

            HelpFileUtil.ShowHelp(helpTopic);
        }

        private void HelpButton_OnClick(object sender, RoutedEventArgs e)
        {
            ShowConextBasedHelp();
        }

        private void AuthorTextBox_OnTextChanged(object sender, TextChangedEventArgs e)
        {
            AuthorPreviewText.Text = _tokenReplacer.ReplaceTokens(AuthorTextBox.Text);
        }

        private void TitleTextBox_OnTextChanged(object sender, TextChangedEventArgs e)
        {
            TitlePreviewText.Text = _tokenReplacer.ReplaceTokens(TitleTextBox.Text);
        }

        private void AuthorTokensComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InsertToken(AuthorTextBox, (ComboBox)sender);
        }

        private void TitleTokensComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InsertToken(TitleTextBox, (ComboBox)sender);
        }

        private void FilenameTokensComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InsertToken(FilenameTemplateTextBox, (ComboBox)sender);
        }

        private void TargetFolderTokensComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InsertToken(TargetFolderTextBox, (ComboBox)sender); //!!!!!!!!!!!!!!!!!!!!
        }

        private void InsertToken(TextBox textBox, ComboBox comboBox)
        {
            var text = comboBox.Items[comboBox.SelectedIndex] as string;
            // use binding-safe way to insert text with settings focus
            TextBoxHelper.InsertText(textBox, text);
        }

        private void UIElement_OnLostFocus(object sender, RoutedEventArgs e)
        {
            try
            {
                int quality = Convert.ToInt32(JpegQualityTextBox.Text);
                if (quality < 1)
                    JpegQualityTextBox.Text = "1";
                if (quality > 100)
                    JpegQualityTextBox.Text = "100";
            }
            catch (Exception)
            {
                JpegQualityTextBox.Text = "75";
            }
        }

        private string QueryProfileName(string profileName)
        {
            var f = new InputBoxWindow();
            f.IsValidInput = ViewModel.ProfilenameIsValid;
            f.QuestionText = TranslationUtil.GetTranslation("ProfileSettingsWindow", "EnterProfileName", "Please enter profile name:");

            if (profileName != null)
                f.InputText = profileName;
            else
                f.InputText = TranslationUtil.GetTranslation("ProfileSettingsWindow", "NewProfile", "New Profile");

            f.Title = TranslationUtil.GetTranslation("ProfileSettingsWindow", "ProfileName", "Profile name");

            if (f.ShowDialog() != true)
                return null;

            return f.InputText;
        }

        private bool QueryDeleteProfile()
        {
            var message = TranslationUtil.GetFormattedTranslation("ProfileSettingsWindow", "ReallyDeleteProfile",
                "Do you really want to the delete '{0}'?", ViewModel.CurrentProfile.Name);
            var caption = TranslationUtil.GetTranslation("ProfileSettingsWindow", "DeleteProfile",
                "Delete profile");
            if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Question) != MessageWindowResponse.Yes)
            {
                return false;
            }
            return true;
        }

        private bool QueryDeleteProfileWithPrinterMapping(string profileName, string printerName)
        {
            var message = TranslationUtil.GetFormattedTranslation("ProfileSettingsWindow", "DeleteProfileWithMappedPrinter",
                    "The profile '{0}' is assigned to the printer '{1}'. If you delete this profile, the printer will use the default profile instead. Do you want to proceed?", profileName, printerName);
            var caption = TranslationUtil.GetTranslation("ProfileSettingsWindow", "ProfileHasPrinterTitle",
                "Printer assignment");
            if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Question) != MessageWindowResponse.Yes)
            {
                return false;
            }
            return true;
        }

        private void ProfileSettingsWindow_OnLoaded(object sender, RoutedEventArgs e)
        {
            TranslationUtil.TranslatorInstance.Translate(this);

            ViewModel.CurrentProfilePropertyChanged();
            
            SelectFirstEnabledOrFirstAction();
        }

        private static bool QueryDiscardUnsavedChanges()
        {
            var caption = TranslationUtil.GetTranslation("ProfileSettingsWindow", "UnsavedChanges", "Unsaved changes");
            var message = TranslationUtil.GetTranslation("ProfileSettingsWindow", "ReallyWantToCancel", "Do you really want to cancel the settings and discard the changes?");
                
            var response = MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.YesNo, MessageWindowIcon.Question);
            
            return (response == MessageWindowResponse.Yes);
        }

        private static bool QueryDiscardChangesInOtherProfiles()
        {
            var message = TranslationUtil.GetTranslation("ProfileSettingsWindow", "YouMadeChangesInOtherProfiles",
                "You have made changes in at least one other profile.\r\nIf you proceed they will get lost.");
            var caption = TranslationUtil.GetTranslation("ProfileSettingsWindow", "UnsavedChangesInOtherProfiles",
                "Unsaved changes in other profiles");
            if (MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OKCancel, MessageWindowIcon.Warning) == MessageWindowResponse.OK)
                return true;

            return false;
        }

        bool QueryIgnoreDefectiveProfiles(ActionResultDict actionResultDict)
        {
            return DefectiveProfilesWindow.ShowDialogTopMost(actionResultDict);
        }

        private void ProfilesBoxUpdateLayout()
        {
            var item = ProfilesBox.SelectedItem;
            ProfilesBox.SelectedItem = null;
            ProfilesBox.SelectedItem = item;
        }

        private void StampChooseColorButton_OnClick(object sender, RoutedEventArgs e)
        {
            System.Windows.Forms.ColorDialog colorDialog = new System.Windows.Forms.ColorDialog();
            colorDialog.Color = ViewModel.CurrentProfile.Stamping.Color;

            if (colorDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                ViewModel.CurrentProfile.Stamping.Color = colorDialog.Color;
                ViewModel.CurrentProfilePropertyChanged();
                //btnStampingColor.BackColor = _currentProfile.Stamping.Color;
            }
        }

        private void StampChooseFontButton_OnClick(object sender, EventArgs e)
        {
            System.Windows.Forms.FontDialog fontDialog = new System.Windows.Forms.FontDialog();
            fontDialog.Font = new System.Drawing.Font(ViewModel.CurrentProfile.Stamping.FontName, ViewModel.CurrentProfile.Stamping.FontSize);

            try
            {
                if (fontDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    string postScriptName = FontHelper.FindPostScriptName(fontDialog.Font.Name);

                    if (postScriptName == null)
                    {
                        string message = TranslationUtil.GetTranslation("ProfileSettingsWindow", "FontFileNotSupported",
                            "The selected font is not supported. Please select a different font.");

                        throw new InvalidOperationException(message);
                    }

                    ViewModel.CurrentProfile.Stamping.FontName = fontDialog.Font.Name;
                    ViewModel.CurrentProfile.Stamping.PostScriptFontName = postScriptName;
                    ViewModel.CurrentProfile.Stamping.FontSize = fontDialog.Font.Size;

                    UpdateFontLabel(ViewModel.CurrentProfile.Stamping);
                }
            }
            catch (Exception ex)
            {
                if (ex is ArgumentException || ex is InvalidOperationException)
                {
                    string message = TranslationUtil.GetTranslation("ProfileSettingsWindow", "FontFileNotSupported",
                        "The selected font is not supported. Please select a different font.");
                    MessageWindow.ShowTopMost(message, "PDFCreator", MessageWindowButtons.OK, MessageWindowIcon.Warning);
                    return;
                }
                throw;
            }
        }

        private void UpdateFontLabel(Stamping stamping)
        {
            var fontSize = stamping.FontSize.ToString();
            var fontstring = string.Format("{0} {1}pt", stamping.FontName, fontSize);
            if (fontstring.Length > 20)
            {
                fontstring = string.Format("{0}. {1}pt", stamping.FontName.Substring(0, 20 - 4 - fontSize.Length), fontSize);
            }
            StampFontButton.Content = fontstring;
        }

        private void AutoSaveBrowseFolderButton_OnClick(object sender, RoutedEventArgs e)
        {
            var folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();

            folderBrowserDialog.Description = TranslationUtil.GetTranslation("ProfileSettingsWindow", "SelectAutoSaveFolder", "Select folder for automatic saving");
            folderBrowserDialog.ShowNewFolderButton = true;

            if (folderBrowserDialog.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                return;

            // use binding-safe way to insert text with settings focus
            TextBoxHelper.SetText(TargetFolderTextBox, folderBrowserDialog.SelectedPath);
        }

        private void CertificationFileButton_OnClick(object sender, RoutedEventArgs e)
        {
            string title = TranslationUtil.GetTranslation("ProfileSettingsWindow", "SelectCertFile", "Select certificate file");
            string filter = TranslationUtil.GetTranslation("ProfileSettingsWindow", "PfxP12Files", "PFX/P12 files")
                            + @" (*.pfx, *.p12)|*.pfx;*.p12|"
                            + TranslationUtil.GetTranslation("ProfileSettingsWindow", "AllFiles", "All files")
                            + @" (*.*)|*.*";

            FileDialogHelper.ShowSelectFileDialog(CertificationFileTextBox, title, filter);
        }

        private void DefaultTimeServerButton_OnClick(object sender, RoutedEventArgs e)
        {
            TextBoxHelper.SetText(TimeServerUrlTextBox, @"http://timestamp.globalsign.com/scripts/timstamp.dll");
            SecuredTimeserverCheckBox.IsChecked = false;
        }

        private void SecurityPasswordsButton_OnClick(object sender, RoutedEventArgs e)
        {
            bool askUserPassword = ViewModel.CurrentProfile.PdfSettings.Security.RequireUserPassword;

            var pwWindow = new EncryptionPasswordsWindow(EncryptionPasswordMiddleButton.Remove, true, askUserPassword);
            pwWindow.OwnerPassword = ViewModel.CurrentProfile.PdfSettings.Security.OwnerPassword;
            pwWindow.UserPassword = ViewModel.CurrentProfile.PdfSettings.Security.UserPassword;

            pwWindow.ShowDialogTopMost();

            if (pwWindow.Response == EncryptionPasswordResponse.OK)
            {
                ViewModel.CurrentProfile.PdfSettings.Security.OwnerPassword = pwWindow.OwnerPassword;
                ViewModel.CurrentProfile.PdfSettings.Security.UserPassword = pwWindow.UserPassword;
            }
            else if (pwWindow.Response == EncryptionPasswordResponse.Remove)
            {
                ViewModel.CurrentProfile.PdfSettings.Security.UserPassword = "";
                ViewModel.CurrentProfile.PdfSettings.Security.OwnerPassword = "";
            }
        }

        private void SignaturePasswordButton_OnClick(object sender, RoutedEventArgs e)
        {
            var certificationFile = CertificationFileTextBox.Text;

            if (!File.Exists(certificationFile))
            {
                var message = TranslationUtil.GetTranslation("ProfileSettingsWindow", "CertificateDoesNotExist",
                    "The certificate file does not exist.");
                var caption = TranslationUtil.GetTranslation("ProfileSettingsWindow", "PDFSignature",
                    "PDF Signature");
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                return;
            }

            var pwWindow = new SignaturePasswordWindow(PasswordMiddleButton.Remove, certificationFile);
            pwWindow.Password = ViewModel.CurrentProfile.PdfSettings.Signature.SignaturePassword;

            pwWindow.ShowDialog();
            if (pwWindow.SignaturePasswordViewModel.Result == SignaturePasswordResult.StorePassword)
            {
                ViewModel.CurrentProfile.PdfSettings.Signature.SignaturePassword = pwWindow.Password;
            }
            else if (pwWindow.SignaturePasswordViewModel.Result == SignaturePasswordResult.RemovePassword)
            {
                ViewModel.CurrentProfile.PdfSettings.Signature.SignaturePassword = "";
            }
        }

        private void TimeServerLoginButton_OnClick(object sender, RoutedEventArgs e)
        {
            var pwWindow = new TimeServerPasswordWindow();
            pwWindow.TimeServerLoginName = ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerLoginName;
            pwWindow.TimeServerPassword = ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerPassword;

            pwWindow.ShowDialogTopMost();

            if (pwWindow.Response == TimeServerPasswordResponse.OK)
            {
                ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerLoginName = pwWindow.TimeServerLoginName;
                ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerPassword = pwWindow.TimeServerPassword;
            }
            else if (pwWindow.Response == TimeServerPasswordResponse.Remove)
            {
                ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerLoginName = "";
                ViewModel.CurrentProfile.PdfSettings.Signature.TimeServerPassword = "";
            }
        }

        private void ProfileSettingsWindow_OnClosing(object sender, CancelEventArgs e)
        {
            if (ViewModel.ButtonClickedToClose == ButtonClickedToClose.None)
                if (!ViewModel.CheckForClosingWindowAndRestoreTheSettings())
                {
                    e.Cancel = true;
                    return;
                }
            SettingsHelper.ApplySettings(Settings);
            SettingsHelper.SaveSettings();
        }

        private void SaveDialogFolderTokensComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InsertToken(SaveDialogFolderTextBox, (ComboBox)sender);
        }

        private void SaveDialogDirectoryBrowseFolderButton_OnClick(object sender, RoutedEventArgs e)
        {
            var folderBrowserDialog = new System.Windows.Forms.FolderBrowserDialog();

            folderBrowserDialog.Description = TranslationUtil.GetTranslation("ProfileSettingsWindow", "SelectSaveDialogFolder", "Select save dialog folder");
            folderBrowserDialog.ShowNewFolderButton = true;

            if (folderBrowserDialog.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                return;

            TextBoxHelper.SetText(SaveDialogFolderTextBox, folderBrowserDialog.SelectedPath);
        }

        private void JpegFactorTextBox_OnTextChanged(object sender, TextChangedEventArgs e)
        {
            var cursorPosition = JpegFactorTextBox.SelectionStart;
            JpegFactorTextBox.Text = JpegFactorTextBox.Text.Replace(',', '.');
            JpegFactorTextBox.SelectionStart = cursorPosition;
        }
    }
}
