﻿using System;
using System.Threading;
using NLog;
using pdfforge.PDFCreator.Core;
using pdfforge.PDFCreator.Core.Jobs;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Core.Actions;
using pdfforge.PDFCreator.Core.Ghostscript;
using pdfforge.PDFCreator.Core.Settings.Enums;
using pdfforge.PDFCreator.Exceptions;
using pdfforge.PDFCreator.Helper;
using pdfforge.PDFCreator.Threading;
using pdfforge.PDFCreator.Views;
using pdfforge.PDFProcessing;

namespace pdfforge.PDFCreator.Workflow
{
    /// <summary>
    /// Defines the different stats the workflow can be in
    /// </summary>
    public enum WorkflowStep
    {
        Init,
        SelectTarget,
        SetSecurityPasswords,
        SetSignaturePassword,
        SetSmtpPassword,
        SetFtpPassword,
        Convert,
        AbortedByUser,
        Finished
    }

    /// <summary>
    /// The ConversionWorkflow class handles all user interaction that is required to convert a PostScript file.
    /// </summary>
    public abstract class ConversionWorkflow
    {
        private const string MinGsVersion = "9.10";

        /// <summary>
        /// Settings for the conversion process
        /// </summary>
        public PdfCreatorSettings Settings { get; set; }
        
        /// <summary>
        /// JobInfo of the current job
        /// </summary>
        public IJobInfo JobInfo { get; protected set; }

        /// <summary>
        /// The job that is created during the workflow
        /// </summary>
        public IJob Job { get; protected set; }

        /// <summary>
        /// The step the workflow currently is in
        /// </summary>
        public WorkflowStep WorkflowStep { get; protected set; }

        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        /// <summary>
        /// Query and set the location where the files will be created. This may include showing a Dialog to the user, if appropriate.
        /// 
        /// The implementation must set Job.JobInfo.OutputFilenameTemplate and Job.JobInfo.OutputFormat
        /// </summary>
        protected abstract void QueryTargetFile();

        /// <summary>
        /// Query and set passwords for PDF encryption. This may include showing a Dialog to the user, if appropriate.
        /// 
        /// The implementation must set Job.PdfPasswords.OwnerPassword and Job.PdfPasswords.UserPassword
        /// </summary>
        protected abstract bool QueryEncryptionPasswords();

        /// <summary>
        /// Query and set passwords for the PDF signature. This may include showing a Dialog to the user, if appropriate.
        /// 
        /// The implementation must set Job.PdfPasswords.SignaturePassword
        /// </summary>
        protected abstract bool QuerySignaturePassword();

        /// <summary>
        /// Get the delegate that handles the requests to query the mail password during the convesion
        /// </summary>
        /// <returns>A QueryMailPassword delegate that handles everthing</returns>
        protected abstract bool QueryEmailSmtpPassword();

        /// <summary>
        /// Get the delegate that handles the requests to query the mail password during the convesion
        /// </summary>
        /// <returns>A QueryMailPassword delegate that handles everthing</returns>
        protected abstract bool QueryFtpPassword();

        /// <summary>
        /// If the Job failed, this method is called to notify the user about the error
        /// </summary>
        protected abstract void NotifyUserAboutFailedJob();

        protected bool Cancel;

        /// <summary>
        /// Runs all steps and user interaction that is required durign the conversion
        /// </summary>
        public void RunWorkflow()
        {
            try
            {
                DoWorkflowWork();
            }
            catch (ProcessingException ex)
            {
                Logger.Error("Error " + ex.ErrorCode + ": " + ex.Message);
                EvaluateActionResult(new ActionResult(ex.ErrorCode));
            }
            catch (ManagePrintJobsException)
            {
                throw;
            }
            catch (Exception ex)
            {
                Logger.Error(ex);
            }
        }

        private void DoWorkflowWork()
        {
            WorkflowStep = WorkflowStep.Init;

            Logger.Debug("Starting conversion...");

            // Find a Ghostscript version
            bool gsReady = false;
            GhostscriptVersion gsVersion = GhostscriptVersion.GetBestGhostscriptInstance(MinGsVersion);
            GhostScript.UseGhostscriptExe = true;
            if (gsVersion != null)
                gsReady = gsVersion.LoadGhostscript();

            if (!gsReady)
            {
                Logger.Debug("No valid Ghostscript version found. Exiting...");
                var message = TranslationUtil.GetTranslation("ConversionWorkflow", "NoSupportedGSFound",
                    "Can't find a supported Ghostscript installation.\r\n\r\nProgram exiting now.");
                const string caption = @"PDFCreator";
                MessageWindow.ShowTopMost(message, caption, MessageWindowButtons.OK, MessageWindowIcon.Error);
                Environment.Exit(1);
            }

            Logger.Debug("Ghostscript Version: " + gsVersion.Version + " loaded from " + gsVersion.DllPath);

            Metadata originalMetadata = JobInfo.Metadata.Copy();

            Job.ApplyMetadata();

            Job.JobInfo.Metadata.Author = Job.TokenReplacer.ReplaceTokens(Job.Profile.AuthorTemplate);
            Job.JobInfo.Metadata.Title = Job.TokenReplacer.ReplaceTokens(Job.Profile.TitleTemplate);

            Job.OnEvaluateActionResult += EvaluateActionResult;
            Job.OnRetypeSmtpPassword += RetypeSmtpPassword;
            Job.OnRecommendPdfArchitect += RecommendPdfArchitect;
            
            WorkflowStep = WorkflowStep.SelectTarget;
            Logger.Debug("Querying the place to save the file");

            try
            {
                QueryTargetFile();
            }
            catch (ManagePrintJobsException)
            {
                // revert metadata changes and rethrow exception
                JobInfo.Metadata = originalMetadata;
                throw;
            }

            if (Cancel)
            {
                Job.CleanUp();
                return;
            }

            ActionResult preCheck = ProfileChecker.ProfileCheck(Job.Profile);
            if (!EvaluateActionResult(preCheck))
            {
                Job.CleanUp();
                return;
            }

            Logger.Debug("Output filename template is: {0}", Job.OutputFilenameTemplate);
            Logger.Debug("Output format is: {0}", Job.Profile.OutputFormat);

            if (SetActions()) return;

            WorkflowStep = WorkflowStep.Convert;

            Logger.Info("Converting " + Job.OutputFilenameTemplate);

            if (Job.Profile.ShowProgress)
                ShowConversionProgress();

            Job.RunJob();

            if (!Job.Success)
            {
                NotifyUserAboutFailedJob();
            }

            WorkflowStep = WorkflowStep.Finished;
            Job.CleanUp();
        }

        private bool SetActions()
        {
            // Skip Security and Signature if OutputFormat is not PDF
            if ((Job.Profile.OutputFormat == OutputFormat.Pdf) 
                || (Job.Profile.OutputFormat == OutputFormat.PdfA)
                || (Job.Profile.OutputFormat == OutputFormat.PdfX))
            {
                if (Job.Profile.PdfSettings.Security.Enabled)
                {
                    WorkflowStep = WorkflowStep.SetSecurityPasswords;

                    Logger.Debug("Querying encryption passwords");
                    if (!QueryEncryptionPasswords() || Cancel)
                    {
                        Logger.Warn("Cancelled setting encrytpion passwords. No PDF will be created.");
                        Job.CleanUp();
                        return true;
                    }
                }

                if (Job.Profile.PdfSettings.Signature.Enabled)
                {
                    WorkflowStep = WorkflowStep.SetSignaturePassword;

                    Logger.Debug("Querying signature password");
                    if (!QuerySignaturePassword() || Cancel)
                    {
                        Logger.Error("Cancelled setting password for the digital signature. No PDF will be created.");
                        Job.CleanUp();
                        return true;
                    }
                }
            }

            if (Job.Profile.EmailSmtp.Enabled)
            {
                WorkflowStep = WorkflowStep.SetSmtpPassword;

                Logger.Debug("Querying smtp password");
                if (!QueryEmailSmtpPassword() || Cancel)
                {
                    Logger.Error("Cancelled setting password for email over smtp. No PDF will be created.");
                    Job.CleanUp();
                    return true;
                }
            }

            if (Job.Profile.Ftp.Enabled)
            {
                WorkflowStep = WorkflowStep.SetFtpPassword;

                Logger.Debug("Querying ftp password");
                if (!QueryFtpPassword() || Cancel)
                {
                    Logger.Error("Cancelled setting password for email over smtp. No PDF will be created.");
                    Job.CleanUp();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Show something to the user (if desired) while the conversion is going on
        /// </summary>
        protected virtual void ShowConversionProgress()
        {
            var progressWindowThread = new SynchronizedThread(ShowConversionProgressDialog);
            progressWindowThread.SetApartmentState(ApartmentState.STA);

            progressWindowThread.Name = "ProgressForm";

            ThreadManager.Instance.StartSynchronizedThread(progressWindowThread);
        }

        [STAThread]
        private void ShowConversionProgressDialog()
        {
            var conversionStatusForm = new ConversionProgressWindow();

            conversionStatusForm.ApplyJob(Job);
            TopMostHelper.ShowDialogTopMost(conversionStatusForm, true);
        }

        /// <summary>
        /// Create a Workflow based on the job and settings objects provided. This will create an AutoSave Workflow if the job's printer has an AutoSave profile associated or the default profile uses AutoSave.
        /// Otherwise, an interactive workflow will be created.
        /// </summary>
        /// <param name="jobInfo">The jobinfo used for the decision</param>
        /// <param name="settings">The settings used for the decision</param>
        /// <returns>A ConversionWorkflow either for AutoSave or interactive use</returns>
        public static ConversionWorkflow CreateWorkflow(IJobInfo jobInfo, PdfCreatorSettings settings)
        {
            Logger.Trace("Creating Workflow");

            ConversionProfile preselectedProfile = PreselectedProfile(jobInfo, settings);

            Logger.Debug("Profile: {0} (GUID {1})", preselectedProfile.Name, preselectedProfile.Guid);

            IJob job = new GhostscriptJob(jobInfo, settings.ApplicationSettings, preselectedProfile);
            job.AutoCleanUp = true;

            if (preselectedProfile.AutoSave.Enabled)
            {
                Logger.Trace("Creating AutoSaveWorkflow");
                return new AutoSaveWorkflow(job, settings);
            }
            else
            {
                Logger.Trace("Creating InteractiveWorkflow");
                return new InteractiveWorkflow(job, settings);
            }
        }

        /// <summary>
        /// Determines the preselected profile for the printer that was used while creating the job
        /// </summary>
        /// <param name="jobInfo">The jobinfo used for the decision</param>
        /// <param name="settings">The settings used for the decision</param>
        /// <returns>The profile that is associated with the printer or the default profile</returns>
        private static ConversionProfile PreselectedProfile(IJobInfo jobInfo, PdfCreatorSettings settings)
        {
            foreach (var mapping in settings.ApplicationSettings.PrinterMappings)
            {
                if (mapping.PrinterName.Equals(jobInfo.SourceFiles[0].PrinterName, StringComparison.CurrentCultureIgnoreCase))
                {   
                    ConversionProfile p = settings.GetProfileByGuid(mapping.ProfileGuid);
                    if (p != null)
                        return p;
                    break;
                }
            }

            ConversionProfile lastUsedProfile = settings.GetLastUsedProfile();
            if (lastUsedProfile != null)
                return lastUsedProfile;

            // try default profile
            ConversionProfile defaultProfile = SettingsHelper.GetDefaultProfile(settings.ConversionProfiles);
            if (defaultProfile != null)
                return defaultProfile;

            // last resort: first profile from the list
            return settings.ConversionProfiles[0];
        }
     
        /// <summary>
        /// Function to evaluate action result with the contained succes status and error code.
        /// </summary>
        /// <param name="actionResult">action result</param>
        /// <returns>true if proceed with further actions, else cancel process</returns>
        protected abstract bool EvaluateActionResult(ActionResult actionResult);

        protected abstract void RetypeSmtpPassword(object sender, QueryPasswordEventArgs e);

        protected abstract void RecommendPdfArchitect(object sender, EventArgs e);
    }
}
