﻿using System;
using System.IO;
using iTextSharp.text;
using iTextSharp.text.pdf;
using NLog;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Core.Settings.Enums;

namespace pdfforge.PDFProcessing
{
    class BackgroundAdder
    {
        //ActionId = 17;
        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        /// <summary>
        /// Add a backgroundfile (setted in profile) to a document, that is opend in the stamper.
        /// It considers cover- and attachment pages (that can also request a background).
        /// The function does nothing, if background settings are disabled.
        /// </summary>
        /// <param name="stamper">Stamper with document</param>
        /// <param name="profile">Profile with backgroundpage settings</param>
        /// <exception cref="ProcessingException">In case of any error.</exception>
        public static void AddBackground(PdfStamper stamper, ConversionProfile profile)
        {
            try
            {
                DoAddBackground(stamper, profile);
            }
            catch (ProcessingException)
            {
                throw;
            }
            catch (Exception ex)
            {
                throw new ProcessingException(ex.GetType() +  " while addding background:" + Environment.NewLine + ex.Message, 17999);
            }
        }
        
        private static void DoAddBackground(PdfStamper stamper, ConversionProfile profile)
        {
            if (!profile.BackgroundPage.Enabled)
                return;

            Logger.Debug("Start adding background.");

            //var pdfReader = new PdfReader(sourceFilename);
            //int nFile = pdfReader.NumberOfPages;
            int nFile = stamper.Reader.NumberOfPages;

            var backgroundPdfReader = new PdfReader(profile.BackgroundPage.File);
            Logger.Debug("BackgroundFile: " + Path.GetFullPath(profile.BackgroundPage.File));
            int nBackground = backgroundPdfReader.NumberOfPages;

            int startOffset; //number of front pages without background 
            try
            {
                startOffset = StartOffsetAccordingToCover(profile);
            }
            catch (Exception ex)
            {
                throw new ProcessingException(ex.GetType() +  " while detecting number of pages of the cover file: " + profile.AttachmentPage.File + "\r\n" + ex, 17200);
            }

            int endPage; //last page with background
            try
            {
                endPage = nFile - EndOffsetAccordingToAttachment(profile);
            }
            catch (Exception ex)
            {
                throw new ProcessingException("Exception while detecting number of pages of the attachment file: " + profile.AttachmentPage.File + "\r\n" + ex, 17201);
            }

            for (int i = 1; i <= nFile; i++)
            {
                int backgroundPage = GetBackgroundPageNumber(i, nBackground, profile.BackgroundPage.Repetition, startOffset, endPage);
                if (backgroundPage < 1)
                    continue;
                PdfImportedPage page = stamper.GetImportedPage(backgroundPdfReader, backgroundPage);
                PdfContentByte background = stamper.GetUnderContent(i);

                Rectangle wmsize = backgroundPdfReader.GetPageSize(backgroundPage);
                Rectangle pdfsize = stamper.Reader.GetPageSize(i);

                if ((stamper.Reader.GetPageRotation(i) == 90) || (stamper.Reader.GetPageRotation(i) == 270))
                {
                    background.AddTemplate(page, 0, -1f, 1f, 0, ((pdfsize.Height - wmsize.Height) / 2),
                        wmsize.Width + (pdfsize.Width - wmsize.Width) / 2); //270°
                    Logger.Debug("BackgroundPage " + backgroundPage + " turned right, because of automatic rotation of page " + i);
                }
                else if ((pdfsize.Height < pdfsize.Width) && (wmsize.Height > wmsize.Width))
                {
                    //pdfsize.height and width must be changed for this calculation, because the page was rotated     
                    background.AddTemplate(page, 0, -1f, 1f, 0, (pdfsize.Width - wmsize.Height) / 2,
                        wmsize.Width + ((pdfsize.Height - wmsize.Width) / 2)); //270°
                    Logger.Debug("BackgroundPage " + backgroundPage + " turned right to fit better in page " + i);
                }
                else
                    background.AddTemplate(page, (pdfsize.Width - wmsize.Width) / 2, (pdfsize.Height - wmsize.Height) / 2);
                //0° 
            }

            //                              cos  sin  -sin  cos  dx  dy
            //background.AddTemplate(page,  0,   1f,  -1f,   0,  0,  0 ); //90°
            //background.AddTemplate(page, -1f,   0,    0, -1f,  0,  0 ); //180°
            //background.AddTemplate(page,  0,  -1f,   1f,   0,  0,  0 ); //270°
            //background.AddTemplate(page,  1f,   0,    0,  1f,  0,  0 ); //0°
        }

        /// <summary>
        /// Determine the number of pages at the begining of the document, that do not get a background,
        /// considering the coverpage settings in profile. 
        /// Returns 0 if cover is disabled or if "add background to cover" is enabled.
        /// </summary>
        /// <param name="profile">Profile with coverpage settings</param>
        /// <returns>Startpage for Background</returns>
        private static int StartOffsetAccordingToCover(ConversionProfile profile)
        {
            int nCover = 0;
            if (profile.CoverPage.Enabled)
            {
                if (!profile.BackgroundPage.OnCover)
                {
                    var coverPdfReader = new PdfReader(profile.CoverPage.File);
                    nCover = coverPdfReader.NumberOfPages;
                }
            }
            return nCover;
        }

        /// <summary>
        /// Determine the number of pages at the end of the document, that do not get a background,
        /// considering the attachmentpage settings in profile. 
        /// Returns 0 if attachment is disabled or if "add background to attachment" is enabled.
        /// </summary>
        /// <param name="profile">Profile with coverpage settings</param>
        /// <returns>Startpage for Background</returns>
        private static int EndOffsetAccordingToAttachment(ConversionProfile profile)
        {
            int nAttachment = 0;
            if (profile.AttachmentPage.Enabled)
            {
                if (!profile.BackgroundPage.OnAttachment)
                {
                    var attachmentPdfReader = new PdfReader(profile.AttachmentPage.File);
                    nAttachment = attachmentPdfReader.NumberOfPages;
                }
            }
            return nAttachment;
        }


        /// <summary>
        /// Determine the background page for the current document page. 
        /// </summary>
        /// <param name="currentDocumentPage">Current page ot the document</param>
        /// <param name="numberOfBackgroundpages">Number of backgroundpages</param>
        /// <param name="repeat">Repetition of the background</param>
        /// <param name="startOffset">Number of pages that do not get a background at the begining of the document</param>
        /// <param name="endPage">Last page that gets a background</param>
        /// <returns>Number of the backgroundpage or -1 if no background must be set</returns>
        private static int GetBackgroundPageNumber(int currentDocumentPage, int numberOfBackgroundpages, BackgroundRepetition repeat, int startOffset, int endPage)
        {
            if (currentDocumentPage <= startOffset)
                return -1;
            if (currentDocumentPage > endPage)
                return -1;

            switch (repeat)
            {
                case BackgroundRepetition.RepeatAllPages:
                    return (((currentDocumentPage - startOffset - 1) % numberOfBackgroundpages) + 1);
                case BackgroundRepetition.RepeatLastPage:
                    if ((currentDocumentPage - startOffset) < numberOfBackgroundpages)
                        return currentDocumentPage - startOffset;
                    //else if (currentDocumentPage - startOffset >= numberOfBackgroundpages)
                    return numberOfBackgroundpages;
                case BackgroundRepetition.NoRepetition:
                    if ((currentDocumentPage - startOffset) <= numberOfBackgroundpages)
                        return currentDocumentPage - startOffset;
                    //else
                    return -1;
            }
            return -1; //default
        }
    }
}
