﻿using System;
using System.IO;
using iTextSharp.text.pdf;
using NLog;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Core.Settings.Enums;

namespace pdfforge.PDFProcessing
{
    public class PDFProcessor
    {
        //ActionId = 26;
        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        /// <summary>
        /// Check if processing is required.
        /// Therefore the output format must be pdf and security or background page or signing must be enabled.
        /// Furthermore PDF/A requires processing the metadata. 
        /// </summary>
        /// <param name="profile">Profile with PDF settings</param>
        /// <returns>True if processing is required</returns>
        public static bool ProcessingRequired(ConversionProfile profile)
        {
            switch (profile.OutputFormat)
            {
                case OutputFormat.Pdf:
                case OutputFormat.PdfA:
                case OutputFormat.PdfX:
                    return profile.PdfSettings.Security.Enabled
                           || profile.BackgroundPage.Enabled
                           || profile.OutputFormat == OutputFormat.PdfA
                           || profile.PdfSettings.Signature.Enabled;   
            }
            
            return false;
        }

        /// <summary>
        /// Function moves pdfFile to a preprocess file and creates a pdfstamper aiming to the original path.
        /// All requested processing (encryption, add background, update xmp metadata, signing) is done in the stmaper.
        /// Afterwards the stamper gets closed (the original file is now the processed file) and the moved preprocess file gets deleted.
        /// The function does nothing if no processing is required. 
        /// </summary>
        /// <param name="pdfFile">Full path to the PDF file</param>
        /// <param name="profile">Profile with PDF settings</param>
        /// <param name="jobPasswords">Passwords (All required passwords must be set)</param>
        /// <exception cref="ProcessingException">In case of any error</exception>
        public static void ProcessPDF(string pdfFile, ConversionProfile profile, JobPasswords jobPasswords)
        {
            if (!ProcessingRequired(profile))
            {
                Logger.Debug("No processing required.");
                return;
            }
            
            Logger.Debug("Start processing of " + pdfFile);

            if (!File.Exists(pdfFile))
            {
                throw new ProcessingException("File in PdfProcessor does not exist: " + pdfFile, 26100); 
            }

            try
            {
                DoProcessPDF(pdfFile, profile, jobPasswords);
            }
            catch (ProcessingException)
            {
                throw;
            }
            catch (Exception ex)
            {
                Logger.Error(ex.GetType() + " while processing file:" + pdfFile + Environment.NewLine + ex.Message);
                throw new ProcessingException(ex.GetType() + " while processing file:" + pdfFile + Environment.NewLine + ex.Message, 26999); 
            }
        }      

        /// <summary>
        /// Moves original file to preprocess file, which is the original file with an appended "_PrePdfProcessor.pdf".
        /// </summary>
        /// <param name="pdfFile">Full path to PDF file</param>
        /// <returns>Path to preprocess file</returns>
        public static string MoveFileToPreProcessFile(string pdfFile)
        {
            string preProcessFile;
            try
            {
                //create copy of original file 
                preProcessFile = Path.ChangeExtension(pdfFile, "_PrePdfProcessor.pdf").Replace("._", "_"); //.ToUpper().Replace(".PDF", "_PrePdfProcessor.pdf");
                File.Move(pdfFile, preProcessFile);
            }
            catch (Exception ex)
            {
                Logger.Error(ex.GetType() + " while creating pdf pre process file:" + Environment.NewLine + ex.Message);
                throw new ProcessingException(ex.GetType() + " while creating pdf pre process file:" + Environment.NewLine + ex.Message, 26101);
            }

            return preProcessFile;
        }

        private static void DoProcessPDF(string pdfFile, ConversionProfile profile, JobPasswords jobPasswords)
        {
            string preProcessFile = null;
            try
            {
                preProcessFile = MoveFileToPreProcessFile(pdfFile);
                using (PdfStamper stamper = StamperCreator.CreateStamperAccordingToEncryption(preProcessFile, pdfFile, profile,
                    jobPasswords))
                {
                    BackgroundAdder.AddBackground(stamper, profile);
                    XmpMetadataUpdater.UpdateXmpMetadata(stamper, profile);
                    Signer.SignPdfFile(stamper, profile, jobPasswords);
                    stamper.Close();
                }
            }
            finally
            {
                //delete copy of original file
                if (!string.IsNullOrEmpty(preProcessFile))
                    if (File.Exists(preProcessFile))
                        File.Delete(preProcessFile);
            }
        }
    }
}
