﻿using System;
using System.IO;
using System.Text;
using iTextSharp.text.pdf;
using NLog;
using pdfforge.PDFCreator.Core.Settings;
using pdfforge.PDFCreator.Core.Settings.Enums;

namespace pdfforge.PDFProcessing
{
    static class StamperCreator
    {
        //ActionId = 25;
        private static readonly Logger Logger = LogManager.GetCurrentClassLogger();

        /// <summary>
        /// Create a PdfStamper with content of sourcefile aiming to destinationfile.
        /// If encryption is enabled, the stamper will be created with an correspondant signature,
        /// which influences the PDF version. (1.4 for 40bit and 128bit encryption, 1.6 for 128bitAes). 
        /// </summary>
        /// <param name="sourceFile">Full path to the source file</param>
        /// <param name="destinationFile">Full patch to the destination file</param>
        /// <param name="profile">Profile with encryption settings</param>
        /// <param name="jobPasswords">Passwords with owner (and user) password</param>
        /// <returns>Stamper with content of sourcefile stream, aiming to destination file</returns>
        /// <exception cref="ProcessingException">In case of any error</exception>
        public static PdfStamper CreateStamperAccordingToEncryption(string sourceFile, string destinationFile, ConversionProfile profile, JobPasswords jobPasswords)
        {
            PdfStamper stamper;

            try
            {
                stamper = DoCreateStamperAccordingToEncryption(sourceFile, destinationFile, profile, jobPasswords);
            }
            catch (ProcessingException)
            {
                throw;
            }
            catch (Exception ex)
            {
                throw new ProcessingException(ex.GetType() + " while creating stamper:" + Environment.NewLine + ex.Message, 25999);
            }

            return stamper;
        }
        
        private static PdfStamper DoCreateStamperAccordingToEncryption(string sourceFilename, string destinationFilename, ConversionProfile profile, JobPasswords jobPasswords)
        {
            Logger.Debug("Started creating PdfStamper according to Encryption.");
            
            var reader = new PdfReader(sourceFilename);
            var fileStream = new FileStream(destinationFilename, FileMode.Create, FileAccess.Write);
            PdfStamper stamper = null;

            if (!profile.PdfSettings.Security.Enabled)
            {
                if (profile.PdfSettings.Signature.Enabled)
                    stamper = PdfStamper.CreateSignature(reader, fileStream, '\0', null, profile.PdfSettings.Signature.AllowMultiSigning);
                else
                    stamper = new PdfStamper(reader, fileStream, '\0', profile.PdfSettings.Signature.AllowMultiSigning);
            }
            else
            {
                int encryption = CalculatePermissionValue(profile);
                Logger.Debug("Callculated Permission Value: " + encryption);

                if (string.IsNullOrEmpty(jobPasswords.PdfOwnerPassword))
                {
                    Logger.Error("Launched encryption without owner password.");
                    throw new ProcessingException("Launched encryption without owner password.", 25201);
                }

                byte[] ownerPassword = Encoding.Default.GetBytes(jobPasswords.PdfOwnerPassword);

                byte[] userPassword = null;

                if (profile.PdfSettings.Security.RequireUserPassword)
                {
                    if (string.IsNullOrEmpty(jobPasswords.PdfUserPassword))
                    {
                        Logger.Error("Launched encryption without user password.");
                        throw new ProcessingException("Launched encryption without user password.", 25202);
                    }
                    userPassword = Encoding.Default.GetBytes(jobPasswords.PdfUserPassword);
                }
                switch (profile.PdfSettings.Security.EncryptionLevel)
                {
                    case EncryptionLevel.Rc40Bit:
                        if (profile.PdfSettings.Signature.Enabled)
                            stamper = PdfStamper.CreateSignature(reader, fileStream, PdfWriter.VERSION_1_4, null,
                                profile.PdfSettings.Signature.AllowMultiSigning); 
                        else
                            stamper = new PdfStamper(reader, fileStream, PdfWriter.VERSION_1_4,
                                profile.PdfSettings.Signature.AllowMultiSigning);
                        stamper.SetEncryption(userPassword, ownerPassword, encryption, PdfWriter.STRENGTH40BITS);
                        break;
                    case EncryptionLevel.Rc128Bit:
                        if (profile.PdfSettings.Signature.Enabled)
                            stamper = PdfStamper.CreateSignature(reader, fileStream, PdfWriter.VERSION_1_4, null,
                                profile.PdfSettings.Signature.AllowMultiSigning);
                        else
                            stamper = new PdfStamper(reader, fileStream, PdfWriter.VERSION_1_4,
                                profile.PdfSettings.Signature.AllowMultiSigning);
                        stamper.SetEncryption(userPassword, ownerPassword, encryption, PdfWriter.STRENGTH128BITS);
                        break;
                    case EncryptionLevel.Aes128Bit:
                        if (profile.PdfSettings.Signature.Enabled)
                            stamper = PdfStamper.CreateSignature(reader, fileStream, PdfWriter.VERSION_1_6, null,
                                profile.PdfSettings.Signature.AllowMultiSigning);
                        else
                            stamper = new PdfStamper(reader, fileStream, PdfWriter.VERSION_1_6,
                                profile.PdfSettings.Signature.AllowMultiSigning);
                        stamper.SetEncryption(userPassword, ownerPassword, encryption, PdfWriter.ENCRYPTION_AES_128);
                        break;
                    //default:
                    //    break;
                }
            }
            if (stamper == null)
            {
                throw new ProcessingException("PDFStamper could not be created", 25200);   
            }
            return stamper;
        }

        /// <summary>
        /// Calculates the PDF permission value that results in the settings from the given profile
        /// </summary>
        /// <param name="profile">The profile to do the calculations with.</param>
        /// <returns>An integer that encodes the PDF security permissions</returns>
        private static int CalculatePermissionValue(ConversionProfile profile)
        {
            int permissionValue = 0;

            if (profile.PdfSettings.Security.AllowPrinting) permissionValue = permissionValue | PdfWriter.ALLOW_PRINTING;
            if (profile.PdfSettings.Security.AllowToEditTheDocument)
                permissionValue = permissionValue | PdfWriter.ALLOW_MODIFY_CONTENTS;
            if (profile.PdfSettings.Security.AllowToCopyContent) permissionValue = permissionValue | PdfWriter.ALLOW_COPY;
            if (profile.PdfSettings.Security.AllowToEditComments)
                permissionValue = permissionValue | PdfWriter.ALLOW_MODIFY_ANNOTATIONS;

            if ((profile.PdfSettings.Security.EncryptionLevel == EncryptionLevel.Rc128Bit)
                || (profile.PdfSettings.Security.EncryptionLevel == EncryptionLevel.Aes128Bit))
            {
                if (profile.PdfSettings.Security.AllowPrinting)
                    if (profile.PdfSettings.Security.RestrictPrintingToLowQuality)
                        permissionValue = permissionValue ^ (PdfWriter.ALLOW_PRINTING ^ PdfWriter.ALLOW_DEGRADED_PRINTING);
                //Remove higher bit of AllowPrinting
                if (profile.PdfSettings.Security.AllowToFillForms)
                    permissionValue = permissionValue | PdfWriter.ALLOW_FILL_IN; //Set automatically for 40Bit
                if (profile.PdfSettings.Security.AllowScreenReader)
                    permissionValue = permissionValue | PdfWriter.ALLOW_SCREENREADERS; //Set automatically for 40Bit
                if (profile.PdfSettings.Security.AllowToEditAssembly)
                    permissionValue = permissionValue | PdfWriter.ALLOW_ASSEMBLY; //Set automatically for 40Bit
            }
            return permissionValue;
        }
    }
}
